### A Pluto.jl notebook ###
# v0.19.25

using Markdown
using InteractiveUtils

# This Pluto notebook uses @bind for interactivity. When running this notebook outside of Pluto, the following 'mock version' of @bind gives bound variables a default value (instead of an error).
macro bind(def, element)
    quote
        local iv = try Base.loaded_modules[Base.PkgId(Base.UUID("6e696c72-6542-2067-7265-42206c756150"), "AbstractPlutoDingetjes")].Bonds.initial_value catch; b -> missing; end
        local el = $(esc(element))
        global $(esc(def)) = Core.applicable(Base.get, el) ? Base.get(el) : iv(el)
        el
    end
end

# ╔═╡ a145e2fe-a903-11eb-160f-df7ea83fa3e6
begin
	using PlotlyBase, HypertextLiteral, PlutoUI, PlutoPlotly
	using CSV, DataFrames, Dates, PeriodicalDates, NLsolve
end

# ╔═╡ 943edb36-acc8-4b2a-94ab-c544f9eb279b
md"""
# Week 10 - The Zero Lower Bound and Macroeconomic Policy

# Exercises

**Macroeconomics, ISCTE-IUL**
"""

# ╔═╡ 1245aef9-18ce-42f5-af82-c2e6cce70798
md"""
**Vivaldo Mendes, Ricardo Gouveia-Mendes, Luís Clemente-Casinhas; April 2023**
"""

# ╔═╡ 4618a663-f77e-4ccc-9b00-e84dbd5beb34
md"""
### Packages
"""

# ╔═╡ 74de9f66-d5b4-4e0f-a75d-5177b1842191
begin
	function Base.show(io::IO, mimetype::MIME"text/html", plt::PlotlyBase.Plot)
       # Remove responsive flag on the plot as we handle responsibity via ResizeObeserver and leaving it on makes the div flickr during updates
	hasproperty(plt,:config) && plt.config.responsive && (plt.config.responsive = false)   
	show(io,mimetype, @htl("""
			<div>
			<script id=asdf>
			const {plotly} = await import("https://cdn.plot.ly/plotly-2.2.0.min.js")
			const PLOT = this ?? document.createElement("div");
		

		
			Plotly.react(PLOT, $(HypertextLiteral.JavaScript(PlotlyBase.json(plt))));


		
			const pluto_output = currentScript.parentElement.closest('pluto-output')

			const resizeObserver = new ResizeObserver(entries => {
				Plotly.Plots.resize(PLOT)
			})

			resizeObserver.observe(pluto_output)

			invalidation.then(() => {
				resizeObserver.disconnect()
			})
		
			return PLOT
			</script>
			</div>
	"""))
	end
end

# ╔═╡ 0bc2eb14-3b5a-4203-82dd-3ef2044a2c93
md"""
## Exercise 1. Inflation targeting
"""

# ╔═╡ 7e025b57-5962-4ab8-85a0-a3cad37b1be5
Resource("http://ebs.de.iscte.pt/ECB_Rules.png", :width=>600)

# ╔═╡ 5999a598-0408-44f1-bd33-c4d0fa5a6006
md"""

On 8 July 2021, the European Central Bank (ECB) announced a revision of its monetary policy strategy for the first time since its creation in 1998. From the article above in the Financial Times, one can read:

>*"The central bank said its new target of 2 per cent was symmetric, “meaning negative and positive deviations of inflation from the target are equally undesirable”. The new target is a medium-term objective with flexibility to fluctuate in either direction in the short term [and] said it could tolerate temporary moves beyond that point, in a shift that gives policymakers flexibility to keep interest rates at historic lows for longer”.*

a) The file "EZ_inflation.csv" includes data about inflation in the EuroZone (EZ) and Portugal. This data is plotted in Figure 1. Why do you think the ECB stresses "the symmetric deviations of inflation from the target"?

b) Why do you think the article stresses "...in a shift that gives policymakers flexibility to keep interest rates at historic lows for longer”?

c) For Portugal and the Eurozone, what is the mean for the inflation deviations from the target, according to the old strategy (we will call it "Loss1")? And according to the new strategy ("Loss2")?

"""

# ╔═╡ b6f5869c-62af-4e81-ae73-912fe522896e
EZ_Inf = CSV.read("EZ_inflation.csv", delim=';', DataFrame)

# ╔═╡ 7d4589e5-37a5-4803-97bc-24c9a8ea61a5
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ 14a93d0a-2005-4406-9150-319d6583a518
md"""
!!! answer "Answer (b)"

	Here

"""

# ╔═╡ 41c7afb6-a57b-45a7-8b9d-bb30902aab8b
md"""
!!! answer "Answer (c)"

	In the previous figure, we can easily observe why the central bank's loss function is dramatically affected by having only positive deviations from the target as its argument or by having both positive and negative deviations as arguments. The first case can be written as

	$$\text{Loss}=(\pi - \pi^{T})$$ 

	while the second case has to be written as

	$$\text{Loss}=(\pi - \pi^{T})^2$$

	The left-hand panel (Fig. 2) shows the first case, and the loss is minimal.  The right-hand side panel (Fig. 3) shows the second case (where both positive and negative deviations are considered arguments in the loss function with the same weight), and the losses are enormous compared to the left panel. This fact is the fundamental reason the ECB has decided to produce a drastic reform in its monetary policy strategy after more than twenty years since it started to operate.

	The *means* of the Loss function according to each rule (the old rule, which considered only positive deviations from the target, and the new rule, which includes both positive and negative deviations) are presented in the following table. 

	$$\begin{aligned}
	&\text { Table 1: Mean of the Loss function }\\
	&\begin{array}{lcc}
	\hline & \text { Positive deviations } & \text { Both deviations } \\
	\hline \text { Eurozone } & 0.179932 & 0.983776 \\
	\text { Portugal } & 0.508163 & 2.12143 \\
	\hline
	\end{array}
	\end{aligned}$$

"""

# ╔═╡ 0836abe7-5ea6-4189-b783-a2215b6acd72
md"""
## Exercise 2. The Taylor rule
"""

# ╔═╡ 4ce820d4-9eb2-4acd-92c7-820cf46d08ae
md"""

We will use statistical information about the variables that are crucial for the analysis of the Taylor Rule. The data is contained in the file "Taylor.csv". We calculate and plot the Taylor Rule for **three different scenarios**:

- The MP rule, as considered in our textbook, with $\lambda = 0.5, \ \overline{r}=2\%, \pi^T=2\%$. For the calculations, use rates expressed by *percentage points*. This means that $\overline{r}=2, \pi^T=2$, which makes the calculus easier to understand and corresponds to how FRED presents the data. 

- The Taylor Rule with equal weights for the inflation gap and the output gap $(\lambda_{\pi}=\lambda_{y}=0.5)$.

- The (New) Taylor Rule with a higher weight for the output gap: $(\lambda_{\pi}=0.5,\ \lambda_{y}=1.0)$. 

a) Do you consider any of those three rules to explain well the decisions of the central bank of the USA (Fed) concerning the Fed Funds Rate?

b) Can you provide clear evidence that having a Taylor Rule in autopilot may lead to extreme punishments for the economy?

c) Given the evidence in these three rules, are monetary policy rules useless?

"""

# ╔═╡ 66f39956-3665-4259-b10c-a82f3985b54b
Taylor= CSV.read("Taylor.csv", DataFrame)

# ╔═╡ d0e70b80-1e84-412c-9e5f-0161a0b0a935
md"""
Let us insert the data into Pluto
"""

# ╔═╡ 57d7d8ed-dce5-4177-b92b-1114b538814d
begin
	r̄ = 2 ;	λ = 0.5 ; λ_π = 0.5 ; λ_y = 0.5 ; πᵀ = 2; 
	Y = Taylor[:,3] ; Yᴾ = Taylor[:,2] ; π = Taylor[:,4] ; i=Taylor[:,5]
end;

# ╔═╡ 2207f310-b6e0-40ca-9d11-6bd90adbd997
md"""
#### The Textbook rule
"""

# ╔═╡ 1dc9dce6-d373-4eb7-8e30-1030160accb4
Taylor_textbook = r̄ .+ (1+λ) .* π;


# ╔═╡ b0679f0a-69fc-427d-aae5-b3a243ccd407
#savefig(p10_11, "Taylor_textbook.pdf", width=800, height=400, scale=10);

# ╔═╡ 3b119e3e-e9bf-4426-9a4e-509504a7a122
md"""
#### The standard Taylor rule
"""

# ╔═╡ c496f682-73c2-40a8-88bf-c5f8554b9e3c
Taylor_standard = r̄ .+ π .+  λ_π .* (π .- πᵀ) .+ λ_y .* (100 .* ((Y .- Yᴾ) ./ Yᴾ));

# ╔═╡ bfd1d0ef-f5fc-4825-8e17-0be4075e1902


# ╔═╡ cace5be5-7a28-42ea-a958-d8eb5cced10d
md"""
#### The New Taylor Rule: more emphasis on the output-gap
"""

# ╔═╡ 230ed75d-1a16-49ea-9f92-876b69ffad70
md"""
TThe slider below `Δλ_y=0.5` tells us that the central bank is more concerned about the output-gap than in the original version of the rule (where `λ_y=0.5` and `Δλ_y=0`). See what happens by changing the value of this slider.
"""

# ╔═╡ 5a4c70b0-4f7c-4ef3-aabf-af1c46978637
md"""
`Δλ_y = `$(@bind Δλ_y Slider(-0.5:0.5:5.0, default=0.5, show_value=true))
"""

# ╔═╡ 077586e0-70a0-4322-a531-3c71b6eab95f
Taylor_new = r̄ .+ π .+  λ_π .* (π .- πᵀ) .+ (λ_y .+ Δλ_y) .* (100 .* ((Y .- Yᴾ) ./ Yᴾ));

# ╔═╡ b53240b4-4fc7-4b07-acc2-b5381bcd6c86
md"""
!!! tip "Answer (a)"

	Here

"""

# ╔═╡ 87f6de5e-ccbe-4295-a37f-123cfccaf3e8
md"""
!!! tip "Answer (b)"

	Here

"""

# ╔═╡ a6b6892e-c679-46e8-8553-65a3055fb472
md"""
!!! tip "Answer (c)"

	Here

"""

# ╔═╡ 8493e2d6-7a73-4616-aa0b-11d7c8109c23
md"""
## Exercise 3. The trouble with inflation
"""

# ╔═╡ b3f2f75f-ad9a-4e58-bb03-74de28379dbf
md"""
The figure below shows the main contributors to inflation in October 2022 in the USA. The inflation rate for all items was 0.4% between September and October 2022. The monthly inflation rate would correspond to annual inflation of 4.48% if the same 0.4% were verified in all following months. In September, the monthly inflation rate was 0.6%, leading to an annual inflation rate 0f 6.8%.


a) If this trend continues over time, what does it tell us about the behavior of inflation?

b) Looking at the various items in the figure below, what items mainly contributed to the inflation rate in October?

c) Can the Fed do anything to control the prices of those items most responsible for the October inflation?

"""

# ╔═╡ 348c030b-fd65-446c-8620-20a3f561aa81
Resource("https://vivaldomendes.org/images/depot/CPI_October2022.png", :width=>600)

# ╔═╡ afa3ef46-b642-4f62-a1ca-31c4a456cb2b
md"""
!!! tip "Answer (a)"

	Here

"""

# ╔═╡ 470ad9b2-d7be-41f6-9008-49fc4a245b06
md"""
!!! tip "Answer (b)"

	Here

"""

# ╔═╡ fb512cbd-9754-488b-89b7-91ef84dc350c
md"""
!!! tip "Answer (c)"

	Here

"""

# ╔═╡ 5d0cee9b-73c6-4b7e-bb0f-6773be82f845
md"""
## Exercise 4. The Zero Lower Bound
"""

# ╔═╡ c7da461d-1fc4-463e-bd46-900df67d1e03
md"""

Consider the four fundamental functions that allow us to analyze the entire functioning of the economy:

$$\begin{align*}
\text{AD}: \quad Y &=m \cdot \overline{A}-m \cdot \phi \cdot(\overline{r}+\lambda \pi) \\[3mm]           
\text{AS}: \quad  \pi &=\pi^{e}+\gamma\left(Y-Y^{P}\right)+\rho \\[3mm]  
\text{MP}: \quad r&=\overline{r}+\lambda \pi \\[3mm]  
\text{Fisher Eq.}: \quad i& =r +\pi
\end{align*}$$

and the following information concerning exogenous variables and parameters:

$$\begin{aligned}
&\overline{A}=7.6 \ , \ m=2.0 \ , \ \phi=0.2 \ , \ \overline{r}=2.0 \ , \ \lambda=0.5 \ , \ \pi^{e}=2.0 \ , \ \gamma= 4.5 \ , \ Y^{P}=14.0 \ , \ \rho=0.
\end{aligned}$$

Note that the rates $\left\{\pi^{e}, \overline{r}, r, i\right\}$ are measured in percentage points
(e.g., $2.0$ means $2.0\%$), while $\{Y, Y^P , \overline{A}\}$ are measured in trillion dollars. In the solutions below, to avoid doubt, we put the symbol % only at the final step of the solutions.

a) Calculate the short-run equilibrium values for the inflation rate, the level of GDP, the real interest rate, and the nominal interest rate. The graphical representation of this equilibrium is displayed in Fig. 4 and 5.

b) Calculate the inflation rate value that corresponds to the ZLB (Zero Lower Bound).

c) In Fig. 6 and 7, we represent the equilibrium associated with the AS/AD curves graphically (including the part related to the ZLB), and we do the same for the MP curve. Is the equilibrium in the ZLB?

"""

# ╔═╡ ac6cd3a1-f915-4ea9-9294-1158b8b3573f
Resource("http://ebs.de.iscte.pt/AD_AS_ZLB.png", :width=>500)

# ╔═╡ b01f5e40-e584-4615-9fbf-b4e9337649d9
md"""
To simplify the notation, let us write the model as follows:

$$\begin{align*}
\pi_{d}&=\pi_{d} \max -(1 /(m \phi \lambda)) Y \\[3mm]           
\pi_{s}&=\pi_{s} \min +\gamma Y
\end{align*}$$
where:
$\quad \pi_{d} \max= \overline{A}/(\lambda \phi) -\overline{r}/\lambda \quad , 
\quad\pi_{s} \text { min } = \pi^e - \gamma Y^P + \rho.$

"""

# ╔═╡ f0f5635b-897b-4ee0-ab8d-2a5e78b6c7d8
md"""
Now, let us pass the information about parameters, exogenous variables, and the two constants $(\pi_d = \pi_d \text{max} \ , \ \pi_s = \pi_d \text{min})$ into the notebook:
"""

# ╔═╡ a77a222e-aba3-4ac6-a5b0-a7551e3ccce4
begin
		
	Ā3 = 7.6;λ3 = 0.5;m3 = 2.0;	ϕ3 = 0.2;	r̄3 = 2.0 				# Demand	
			
	γ3 = 4.5;	Yᴾ3 = 14.0;	πᵉ3 = 2.0;	ρ3 = 0.0 					# Supply
	
	πd_max3 = (Ā3 / (λ3 * ϕ3)) - r̄3/λ3 								# The AD curve y-axis' intercept 
	
	πs_min3 = πᵉ3 - γ3 * Yᴾ3 + ρ3 									# The AS curve y-axis' intercept
end;

# ╔═╡ c66efe64-1130-42e1-bf43-a7c03baff579
md"""
To answer question **(a)**, we can use the NLsolve package. The initial equilibrium can be calculated as follows:
"""

# ╔═╡ 29089f61-955b-4f02-97a1-9e49ec796e96
begin
	function my_model10_3!(F, v)
	
           	π = v[1]
           	Y = v[2]
		   	r = v[3]
		    i = v[4]
				
           	F[1] = πd_max3 - (1/(m3 * ϕ3 * λ3))*Y - π		# AD curve = 0
           	F[2] = πs_min3 +γ3 * Y - π                 		# AS curve = 0
			F[3] = r̄3 + λ3 * π - r                     		# MP curve = 0 
			F[4] = π + r - i
	end
end;

# ╔═╡ ae6e7a3f-4d2b-417f-aa66-af24a5803896
begin
	solution10_3 = nlsolve(my_model10_3!, [0.0; 0.0; 0.0; 0.0])
	solution10_3.zero
end

# ╔═╡ 6fe378c7-4970-4d12-8106-2ffcba89537d
md"""
!!! tip "Answer (a)"

	Here	

"""

# ╔═╡ e52f724e-4530-4221-92ff-9f558d120db7
md"""
!!! hint

	
	The rate of inflation that corresponds to the ZLB is easy to obtain. We just have to recall that

	$$\pi_{ZL}=-\frac{\overline{r}}{1+\lambda}$$

"""

# ╔═╡ 5652b23c-c152-4a8b-ac7d-b33d782c0800
π_ZL3= - r̄3 / (1+λ3)

# ╔═╡ 34adc2d7-d7a6-4f62-b151-c71eff996074
md"""
!!! tip "Answer (b)"

	Here

"""

# ╔═╡ a006eac2-59aa-43c1-b30a-8f7c2391ef27
md"""
!!! tip "Answer (c)"

	Here

"""

# ╔═╡ c38dbe18-a85a-4960-9c99-674196e0526d
md"""
## Exercise 5. A negative demand shock
"""

# ╔═╡ cb0d6593-5ca4-4afd-bddc-88506c4cf477
md"""

Consider the same values as in exercise $4$. However, there is an external shock that affects the Autonomous Demand, which decreases $\overline{A}$ by \$0.2 trillion. 

a) The short-run equilibrium values for the inflation rate, the level of GDP, and the real interest rate are represented in Figs 10 and 11. What is the level of the nominal interest rate in this equilibrium?

b) Is the economy in a recession, or in an economic boom? Justify.

c) If there is no other demand or supply shock what will happen to the economy over time?

d) What will be the new value of the long-run equilibrium inflation rate? What about the real and nominal interest rates?

"""

# ╔═╡ afcdf0e6-4563-4bbb-a2db-6aa6c2d07b42
md"""
!!! tip "Answer (a)"

	Here

"""

# ╔═╡ 3e0f6675-3586-4f9f-8cec-cf87acae58f6
md"""
!!! tip "Answer (b)"

	Here

"""

# ╔═╡ 46fae038-8319-4f88-b554-600ef7c762c8
md"""
!!! tip "Answer (c)"

	Here

"""

# ╔═╡ b6f43915-f5a4-471e-b018-c8c31d3185b9
md"""
To answer question (d), we have to represent this new equilibrium graphically by using the slider `self_correction4`. By how much will inflation decline such that the new AS function will cross the$Y^P$ curve at $0$%? A quick inspection of the plots above will tell us that the self-correcting mechanism will force inflation to go down by 2 percentage points. We can call this mechanism by "self_correction" 
"""

# ╔═╡ d9715727-6545-4f32-a9f4-f450ee98341d
self_correction4= -2

# ╔═╡ d9c4f0f6-f54e-4bd8-ba73-0797fbd23dde
md"""
!!! tip "Answer (d)"

	Here

"""

# ╔═╡ add07024-b7eb-445d-b4f4-6a6757c2ecd3
md"""
## Exercise 6. Secular stagnation
"""

# ╔═╡ 7f67d656-bb55-4097-ba49-10140f9b3502
md"""
Suppose the external shock on $\overline{A}$ is huge: e.g., $-0.5$ trillion dollars. Its impact can be seen in the figure below. What will happen in the economy? 

"""

# ╔═╡ 7e47b3bc-a45c-4d84-9a58-d4f01d6ba99f
md"""
`ΔĀ5 = `$(@bind ΔĀ5 Slider(-1.0:0.05:1.0, default=-0.5, show_value=true))
"""

# ╔═╡ bdea4838-cbd9-4947-bc74-b351658952c1
md"""
!!! answer

	Here

"""

# ╔═╡ dcd13560-0f73-4aaa-b59a-b7961d8cdc63
md"""
## Exercise 7. Japanese style deflation trap (difficult problem)
"""

# ╔═╡ 82a2c755-960e-4eb7-a55a-e333eb85fa5f
md"""
Consider the data from Exercise 4. However, due to the falling oil prices in international markets, supply suffers a positive shock of -2.0. Simultaneously, inflation expectations fall by 2 percentage points; the new value for inflation expectations is $\pi^e=0\%$. These impacts can be seen in the figure below. What will happen to the economy if there is no active fiscal policy?
"""

# ╔═╡ 4ac136d2-0413-4b4d-a990-9bf2a49ad482
ρ6 = -2.0 ;	πᵉ6 = 0.0;

# ╔═╡ 5ce15489-49a3-4942-b3dc-15a7629169c6
md"""
#### Solutions
"""

# ╔═╡ 292b1c34-b9d5-4468-98a9-4ee7a85d865b
md"""
A positive supply shock of -2 $(\rho=-2.0)$, with a simultaneous decline in inflation expectations by 2 percentage points $(\pi^e=0\%)$, moved the economy out of point 1. The economy will end up either in point 2 or in point 2zl. The *economic fundamentals* are still there: (i) there is a new AS curve which supplies a certain level of goods & services at a lower level of inflation than before those two events occurred; (ii) the AD curve is the same as before. Why do we have two different short-term macroeconomic equilibria? Because people may behave in strange ways, not easily explained by the "economic fundamentals". 

Suppose most people **believe** that the two exogenous factors will make the economy fall to a significant recession and deflation. In that case, these self-fulfilling prophecies will force the economy to move to point 2zl. At this point, the economy is in recession and, by the market self-correcting mechanism, inflation will have to come down, forcing the AS curve to move to the right until it reaches point 3. This point is a short-term and long-term equilibrium point, which means that the economy will be in a deflationary situation if nothing else happens.  What may take the economy out of this situation if no other shock occurs? The usual tools culprits will do: a significant expansionary fiscal policy or a non-conventional monetary policy.

On the other hand, if most people are not too worried about the negative impact of those two factors, this attitude will have a positive effect on economic activity, and the economy will jump into point 2. Notice that there is nothing terribly wrong with having lower oil prices, neither with private agents formulating lower inflation expectations. What happens is that aggregate demand will increase if inflation goes down, and so will also GDP.  Point 2 is a short-term equilibrium but not a long-term one. As we are in an economic boom, inflation will have to increase due to the market self-correcting mechanism, and the AS curve will move up until it reaches point 1.  This point is both a short-run and a long-run macroeconomic equilibrium. In this case, we do not need an active fiscal policy to deal with the long-run disequilibrium in point 2. A conventional monetary policy has to accommodate the gradual increase in inflation while the economy moves from point 2 to point 1.

In macroeconomics, this situation of having the same economic fundamentals and two different long-term macroeconomic equilibria is normally denominated by "self-fulfilling prophecies", "sunspots", or "indeterminacy". As Isaac Newton once put it: *"I can calculate the motion of heavenly bodies, but not the madness of people"*. The madness of people may prevail everywhere, even in macroeconomics.
"""

# ╔═╡ 26fe310e-9ed8-4d09-bb82-a5bd39193dde
md"""
# Reality Check
"""

# ╔═╡ 63e8392d-c2b3-4a1a-b9f1-2acffe23ecb4
md"""
## Application 1

### An important speech by Jerome Powel, Fed's Chair 
"""

# ╔═╡ 3bed5d29-9973-4990-961a-53e53451854e
md"""

*"Inflation that runs below its desired level can lead to an unwelcome fall in longer-term inflation expectations, which, in turn, can pull actual inflation even lower, resulting in an adverse cycle of ever-lower inflation and inflation expectation.* 

*This dynamic is a problem because expected inflation feeds directly into the general level of interest rates. Well-anchored inflation expectations are critical for giving the Fed the latitude to support employment when necessary without destabilizing inflation. But if inflation expectations fall below our 2 percent objective, interest rates would decline in tandem. In turn, we would have less scope to cut interest rates to boost employment during an economic downturn, further diminishing our capacity to stabilize the economy through cutting interest rates. We have seen this adverse dynamic play out in other major economies around the world and have learned that once it sets in, it can be very difficult to overcome. We want to do what we can to prevent such a dynamic from happening here.” (page 8/9)*

[Jerome Powel (2020)](https://www.federalreserve.gov/newsevents/speech/files/powell20200827a.pdf) *New Economic Challenges and the Fed’s Monetary Policy Review*, speech at the Conference  “Navigating the Decade Ahead:  Implications for Monetary Policy,” symposium sponsored by the Federal Reserve Bank of Kansas City, August 27, 2020. 

Do you see any connection between what we discussed during this week and the great concerns of the President (Chair) of the Federal Reserve Board of the USA in his most relevant speech in 2020? Do you think that the problems of deflation and lower inflation expectations are irrelevant issues in modern macroeconomics?

"""

# ╔═╡ 9c5a5fcc-4655-4250-9214-1e819a862fa5
md"""
	!!! hint

		This speech touches on all major points covered this week: the fear of deflation, the importance of inflation expectations, the relevance of having a 2% target interest rate, and a responsible central should do all it takes to prevent deflation from setting in.


	"""

# ╔═╡ 8bfbcdd2-66b4-4884-8edd-71d3d8416a21
md"""
## Application 2

### Ignore Japan at your peril 
"""

# ╔═╡ 9f0b3172-d9cd-4275-8b20-cb42e988cc0f
Japan= CSV.read("Japan_Deflation.csv", delim=';', DataFrame)

# ╔═╡ a9818a71-fc9c-483a-9482-2f062232e73d
period10_8 = MonthlyDate(1997,10):Month(1):MonthlyDate(2021,5);

# ╔═╡ 6cee0cc7-2695-48df-932d-0128fcae87a3
begin
	
	trace10_80 = scatter(;x = Date.(period10_8), y = Japan[:,2], 
				mode="markers+lines", name  = "UR",
				marker_symbol="circle", marker_size="4",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "Blue")
	
	trace10_81 = bar(;x = Date.(period10_8), y = Japan[:,3], 
				mode="markers+lines", name  = "CPI", 
				marker_symbol="circle", marker_size="4",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "red")
	

	layout10_81 = Layout(;
		
		title_text = "Inflation (CPI) and Unemployment (UR) Rates in Japan: 1997-2021", title_x =0.5,
		
		hovermode="x",
		
		
        xaxis = attr(
               title = "Monthly observations", 
               tickformat = "%Y",
               hoverformat = "%Y-M%m",
               tick0 = "1997/10/01",
               dtick = "M36"),
		
        #xaxis_range = [2000, 2020],
        yaxis_title = "Percentage points",
        #yaxis_range=[-2, 2], 
        titlefont_size = 16)

	p10_82 = Plot([trace10_80, trace10_81], layout10_81)
end

# ╔═╡ ad3c4297-138f-4bdd-b839-e85ead198ee7
md"""
It is incredible how a developed, democratic and open society felt into the deflation trap since June 1998, even though successive governments and central bank's presidents tried to avoid such a situation. The central bank of Japan has an inflation target of 2%, as most developed economies. But since mid 1998, only in 12 out of the 264 months were Japan capable of achieving such a target. In fact, most of the time, Japan has experienced quite severe deflation (-1%, -2%, and even -2.5%), some of them for extended periods. Surprisingly, *despite significant deflation, the Japanese economy has displayed relatively low unemployment rates*. 

Can we explain this Japanese case with our model? And with our exercises above? Yes, Exercise 7 can easily explain the Japanese case. Why has Japan not been able to move to the "good" long-term equilibrium as mentioned in that exercise, stuck in the "bad" one? Some influencial economists have argued that in the past -- before 2013, when Prime Minister Shinzo Abe came to power and nominated a new Governor of the Central Bank of Japan -- Japanese governments and the central bank did not take the necessary measures to avoid the economy diving into the deflationary trap. They only took timid measures to deal with an exceptional danger, and the trap settled in. The positive impact on inflation of the measures taken by prime minister Shinzo Abe and his appointed Bank of Japan Governor (Haruhiko Kuroda) after 2013 can be easily seen in the figure above. Since 2013, the number of months with positive inflation is much greater than the number of those with deflation: a very different picture from what had happened before. 
"""

# ╔═╡ 1503e906-0783-407d-a3f7-134d3c99c766
md"""
# Auxiliary cells (do not delete)
"""

# ╔═╡ a385a0ae-d2e0-4b4e-862c-e69ec5bd6843
html"""<style>
main {
    max-width: 950px;
    align-self: flex-start;
    margin-left: 100px;
}
"""


# ╔═╡ 9fa87191-7c90-4738-a45a-acd929c8bd1b
  TableOfContents()

# ╔═╡ 99e824b2-e12c-4090-87cb-45b3d042ee79
md"""
!!! note "Target inflation rate across the world"

	| Central Bank        | Target          |    | Central Bank          | Target
	| :---------------    | ------------:   |:---| :---------------      | -------:
	| US                  | 2               |    | New Zealand           | 2,$\pm$ 1
	| Japan               | 2               |    |Australia 		    | [2-3] 
	| EuroZone            | 2               |    |Canada                | 2,$\pm$ 1
	| UK                  | 2               |    |Switzerland           | <2
	| Sweden              | 2               |    |China		 			| 3
	| Norway              | 2               |    |Mexico 			 	| 3,$\pm$ 1

""";

# ╔═╡ 916b9c85-e357-4d10-bba1-2b641a4dbc28
md"""
##### Exercise 1. Supporting cells
"""

# ╔═╡ 3d73b76d-59e2-4249-9034-f9bbb2a1074b
period10_1 = MonthlyDate(1997,1):Month(1):MonthlyDate(2021,6);

# ╔═╡ 99fc937d-f798-418a-a57f-5ddc3feb5f3e
begin
	trace10_1 = scatter(;x = Date.(period10_1), y = EZ_Inf[:,2], 
				name = "EuroZone", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3, 
            	marker_color = "Blue")
	
	trace10_2 = scatter(;x = Date.(period10_1), y = EZ_Inf[:,3], 
				name = "Portugal", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3,
            	marker_color = "Red")

	layout10_2 = Layout(;
		
			title_text = "Fig 1. Inflation in the EuroZone and Portugal", title_x=0.5,
		
			hovermode="x",		
		
            xaxis = attr(
               title = "Monthly obervations",
               tickformat = "%Y",
               hoverformat = "%Y-M%m",
               tick0 = "1997/01/01",
               dtick = "M60" ),
		
        	#xaxis_range = [2000, 2020],
        	yaxis_title = "Percentage points",
        	#yaxis_range=[-2, 2], 
        	titlefont_size = 16)

	p10_2 = Plot([trace10_1, trace10_2], layout10_2)
end

# ╔═╡ 2dce7165-029d-43cc-8358-9a8df3f86d2a
Loss2 =(EZ_Inf[:,2:3].-2).^2;

# ╔═╡ 8362fd70-ae19-45f8-bb6d-386946cd793c
describe(Loss2, :mean, :min, :median, :max , :std)

# ╔═╡ 0b858694-3d54-4f05-8132-9bfcd302534a
begin
	trace10_7 = scatter(;x = Date.(period10_1), y =Loss2[:,1], 
				name = "EuroZone", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "Blue")
	
	trace10_8 = scatter(;x = Date.(period10_1), y = Loss2[:,2], 
				name = "Portugal", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "Red")

	layout10_8 = Layout(;
		
			title_text = "Fig 3. Squared Deviations from Target",
			title_x = 0.5,
			
			hovermode="x",
		
            xaxis = attr(
               title = "Monthly observations",
               tickformat = "%Y",
               hoverformat = "%Y-M%m",
               tick0 = "1997/01/01",
               dtick = "M60" ),
		
        	#xaxis_range = [2000, 2020],
        	yaxis_title = "Percentage points",
        	yaxis_range=[0,15], 
        	titlefont_size = 16)


	p10_8 = Plot([trace10_7, trace10_8], layout10_8)
end;

# ╔═╡ dde94490-9fdd-4417-90ae-0a6735eb505a
Loss = EZ_Inf[:,2:3].-2;

# ╔═╡ c9942e2d-318d-48b8-b145-96425791cae5
begin
	trace10_3 = scatter(;x = Date.(period10_1), y =Loss[:,1], 
				name = "EuroZone", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "Blue")
	
	trace10_4 = scatter(;x = Date.(period10_1), y = Loss[:,2], 
				name = "Portugal", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "Red")

	layout10_4 = Layout(;
		
			title_text = "Inflation in the EuroZone and Portugal: Deviations from Target", title_x=0.5,
		
			hovermode="x",
		
            xaxis = attr(
               title = "Monthly obervations",
               tickformat = "%Y",
               hoverformat = "%Y-M%m",
               tick0 = "1997/01/01",
               dtick = "M60" ),
		
        	#xaxis_range = [2000, 2020],
        	yaxis_title = "Percentage points",
        	#yaxis_range=[-2, 2], 
        	titlefont_size = 16)

	p10_4 = Plot([trace10_3, trace10_4], layout10_4)
end;

# ╔═╡ 5811e2cb-4448-4b03-a812-ab284b583c86
begin
	#Loss1 = mapcols(t -> ifelse.(t .< 0, 0, t), Loss)
	Loss1 = ifelse.(Loss .< 0, 0, Loss)
	#Loss1 = clamp.(Loss, 0.0, Inf)
end;

# ╔═╡ f6081970-7d0e-4aba-b712-bca0a06d4319
describe(Loss1, :mean, :min, :median, :max, :std)

# ╔═╡ edf08bd1-dcd2-4129-a61f-c1e4f6bb09d9
begin
	trace10_5 = scatter(;x = Date.(period10_1), y =Loss1[:,1], 
				name = "EuroZone", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "Gray")
	
	trace10_6 = scatter(;x = Date.(period10_1), y = Loss1[:,2], 
				name = "Portugal", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "Orange")

	layout10_6 = Layout(;
		
			title_text = "Fig 2. Positive Deviations from Target",
			title_x = 0.5,
			hovermode="x",
		
            xaxis = attr(
               title = "Monthly observations",
               tickformat = "%Y",
               hoverformat = "%Y-M%m",
               tick0 = "1997/01/01",
               dtick = "M60" ),
		
        	#xaxis_range = [2000, 2020],
        	yaxis_title = "Percentage points",
        	yaxis_range=[0,15], 
        	titlefont_size = 16)

	p10_6 = Plot([trace10_5, trace10_6], layout10_6)
end;

# ╔═╡ 9446db0c-db46-4d42-beca-55f152f0e8f5
begin
p10_9 = [p10_6  p10_8]
end

# ╔═╡ 056d9341-78f1-4cb1-8a8e-a71ff246cf9d
md"""
##### Exercise 2. Supporting cells
"""

# ╔═╡ da6d64b3-cea6-42b4-9270-97e5df74c5cf
period10_2 = QuarterlyDate(1960, 1):Quarter(1):QuarterlyDate(2022,4);

# ╔═╡ 2275faad-ae60-49b6-a2e9-eda7a806b134
begin
	trace10_10 = scatter(;x = Date.(period10_2), y = Taylor_textbook, 
				name = "Textbook Rule", mode="markers+lines",
				marker_symbol="circle", marker_size="4.5",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "Blue")
	
	trace10_11 = scatter(;x = Date.(period10_2), y = i, 
				name = "FFR", mode="markers+lines",
				marker_symbol="circle", marker_size="4.5",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "Red")

		
	layout10_11 = Layout(;
				legend=attr( x=0.8, y=0.95),
		
				title_text = "The textbook rule (USA: 1960.Q1--2022.Q3)", title_x = 0.5,
		
				hovermode="x",		
		
            	xaxis = attr(
                title = "Quarterly obervations",
                tickformat = "%Y",
                hoverformat = "%Y-Q%q",
                #tick0 = "1960/04/01",
                #dtick = "M120",
        				),
        		xaxis_range = [Date.(1959), Date.(2023)],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)
	
	

	p10_11 = Plot([trace10_10, trace10_11], layout10_11)
end

# ╔═╡ 89565b78-3ce4-4f0e-9ee2-0d563d7f9e68
let
	fig= Plot(Layout(Subplots(rows=2, cols=1, 
			subplot_titles=	["Taylor rule versus the Federal Funds Rate" "Taylor rule minus the Federal Funds Rate" ], 
			vertical_spacing=0.1, horizontal_spacing = 0.09)))
	
	add_trace!(fig, scatter(;x = Date.(period10_2), y = (Taylor_standard), mode="markers+lines", marker_size = 4.5, line_width= 0.3, marker_color = "Purple", name = "Taylor Rule"),   row=1, col=1)
	
	add_trace!(fig, scatter(;x = Date.(period10_2), y = i, name = "FFR"), row=1, col=1)

	add_trace!(fig, bar(;x = Date.(period10_2), y = (Taylor_standard .- i), mode="markers+lines", line_width= 0.3, marker_color = "darkblue", name = ""),  row=2, col=1)
	
	relayout!(fig, hovermode="x", height = 700, legend_orientation="h", legend=attr( x=0.65, y=0.97), xaxis_range = [Date.(1960), Date.(2023)],)
	
	PlutoPlotly.plot(fig) # This need to be the `plot` function from PlutoPlotly, not from PlotlyJS
end

# ╔═╡ a7b2ef76-4626-4e89-9c78-98fa286923b3
let
	fig= Plot(Layout(Subplots(rows=2, cols=1, 
			subplot_titles=	["New Taylor rule versus the Federal Funds Rate" "New Taylor rule minus the Federal Funds Rate" ], 
			vertical_spacing=0.1, horizontal_spacing = 0.09)))
	
	add_trace!(fig, scatter(;x = Date.(period10_2), y = (Taylor_new), mode="markers+lines", marker_size = 4.5, line_width= 0.3, marker_color = "blue", name = "New Taylor Rule"),   row=1, col=1)
	
	add_trace!(fig, scatter(;x = Date.(period10_2), y = i, name = "FFR"), row=1, col=1)

	add_trace!(fig, bar(;x = Date.(period10_2), y = (Taylor_new .- i), mode="markers+lines", line_width= 0.3, marker_color = "darkred", name = ""),  row=2, col=1)
	
	relayout!(fig, hovermode="x", height = 700, legend_orientation="h", legend=attr( x=0.65, y=0.97), xaxis_range = [Date.(1960), Date.(2023)],)
	
	PlutoPlotly.plot(fig) # This need to be the `plot` function from PlutoPlotly, not from PlotlyJS
end

# ╔═╡ 18a64d4b-5741-44c8-ac1c-60e00f03828c
md"""
##### Exercise 3. Supporting cells
"""

# ╔═╡ d7e4c204-d21d-4119-a26f-124cf6ec5fa5
begin
	trace10_12 = scatter(;x = Date.(period10_2), y = Taylor_standard, 
				name = "Taylor Rule", mode="markers+lines",
				marker_symbol="circle", marker_size="4.5",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "Blue")
	
	trace10_13 = scatter(;x = Date.(period10_2), y = i, 
				name = "FFR", mode="markers+lines",
				marker_symbol="circle", marker_size="4.5",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "Red")

		
	layout10_13 = Layout(;
	
				legend=attr( x=0.8, y=0.95),
				title_text = "The Taylor Rule: Standard Version (USA: 1960.Q1--2022.Q4)", title_x =0.5,
		
				hovermode="x",		
		
            	xaxis = attr(
                title = "Quarterly obervations",
                tickformat = "%Y",
                hoverformat = "%Y-Q%q",
                #tick0 = "1960/01/01",
                #dtick = "M120",	),
				),
        		xaxis_range = [Date.(1959), Date.(2023)],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)
	
	

	p10_13 = Plot([trace10_12, trace10_13], layout10_13)
	#savefig(p10_13,"C:\\Users\\Utilizador\\Desktop\\AAA_Aulas\\Mishkin_Julia_Files\\coco.pdf")
end;

# ╔═╡ 0ce61106-cc15-44fa-a9a6-02ae2074154a
#savefig(p10_13, "Taylor_standard.pdf", width=800, height=400, scale=10);

# ╔═╡ 6991c328-413a-4947-8542-a585e9010bca
begin
	trace10_14 = bar(;x = Date.(period10_2), y = (Taylor_standard.-i), 
				name = "Taylor Rule", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	#marker_color = "DarkBlue"
	)
	
			
	layout10_14 = Layout(; 
		
				title_text = "Taylor rule minus the Fed Funds Rate (USA: 1960.Q1--2022.Q4)", title_x =0.5,
		
				hovermode="x",
		
            	xaxis = attr(
                title = "Quarterly obervations",
                tickformat = "%Y",
                hoverformat = "%Y-Q%q",
                tick0 = "1960/01/01",
                dtick = "M120",
        				),
        		#xaxis_range = [1960, 2020],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)
	
	

	p10_14 = Plot([trace10_14], layout10_14)
end;

# ╔═╡ f7404380-0681-4466-ac61-a0f1dc4b94ef
begin
	trace10_15 = scatter(;x = Date.(period10_2), y = Taylor_new, 
				name = "New Taylor Rule", mode="markers+lines",
				marker_symbol="circle", marker_size="4.5",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "DarkBlue")
	
	trace10_16 = scatter(;x = Date.(period10_2), y = i, 
				name = "FFR", mode="markers+lines",
				marker_symbol="circle", marker_size="4.5",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "Red")

		
	layout10_16 = Layout(;

				legend=attr( x=0.8, y=0.95),
	
				title_text = "New Taylor rule: more emphasis on the output gap (USA: 1960.Q1-2022.Q4)", 
				title_x =0.5,
		
				hovermode="x",	
		
            	xaxis = attr(
                title = "Quarterly observations",
                tickformat = "%Y",
                hoverformat = "%Y-Q%q",
                #tick0 = "1960/01/01",
                #dtick = "M120",
        				),
        		xaxis_range = [Date.(1959), Date.(2023)],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)
	
	

	p10_16 = Plot([trace10_15, trace10_16], layout10_16)
end;

# ╔═╡ a7d4f49e-e317-44bd-bc77-a136e8182d59
#savefig(p10_16, "Taylor_new.pdf", width=800, height=400, scale=10);

# ╔═╡ 99e3138f-5fea-43e7-8f6b-195f2517c673
begin
	trace10_17 = bar(;x = Date.(period10_2), y = (Taylor_new.-i), 
				name = "Taylor Rule", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3, 
            	#marker_color="LightSteelBlue",
            	marker_color = "darkred")
	
			
	layout10_17 = Layout(;
		
				title_text = "New Taylor rule minus the Federal Funds Rate (USA: 1960.Q1-2022.Q4)", title_x =0.5,
		
				hovermode="x",
		
            	xaxis = attr(
                title = "Quarterly obervations",
                tickformat = "%Y",
                hoverformat = "%Y-Q%q",
                tick0 = "1960/01/01",
                dtick = "M120",
        				),
        		#xaxis_range = [1960, 2020],
        		yaxis_title = "Percentage points",
        		#yaxis_range=[-2, 2], 
        		titlefont_size = 16)

	p10_17 = Plot([trace10_17], layout10_17)
end;

# ╔═╡ c44af435-f632-4542-b914-fe83dcde78a5
md"""
##### Exercise 4. Supporting cells
"""

# ╔═╡ e3be2df9-6d5f-4541-ba35-ec65ebfeb4c3
begin
	Y3 = 12.0:0.01:15.0
	
	πd3 = πd_max3 .- ((1 ./(m3 .* ϕ3 .* λ3)) .* Y3) 
	πs3 = πs_min3 .+ γ3 .*Y3             
	
	trace10_20 = scatter(; x = [14,14] , y = [-8,14] , mode = "line", 
		line_width = "4", name = "Yp", line_color = "Orange")
	
	trace10_21 = scatter(; x = Y3, y = πd3, mode="lines" , 
		line_color = "Blue", line_width = "3", name = "AD")
	
	trace10_22 = scatter(; x = Y3, y = πs3, mode="lines" , 
		line_color = "Red", line_width = "3", name  = "AS")
	
	trace10_23 = scatter(; x = [14.0], y = [2.0], text =["1"], 
		textposition = "top center", name = "Eq1", mode = "markers+text", 
		marker_size = "11", marker_color = "Blue", 
		textfont = attr(family="sans serif", size=18, color="black"))
	
	
	layout10_23 = Layout(;
		title_text ="Fig 5. Macroeconomic Equilibrium",
		title_x = 0.5,
		hovermode = "x",
        xaxis = attr(title=" GDP trillion dollars (Y)", 
			showgrid=true, zeroline=false),
        xaxis_range = [13.5, 14.5],	
        yaxis = attr(title="Rate of inflation (π)", zeroline = false),
		yaxis_range = [-0 , 5])

   p10_23 = Plot([trace10_20, trace10_21, trace10_22, trace10_23], layout10_23)
	
end;

# ╔═╡ cc0f9f16-fd0b-4024-9fe7-744294d744c3
begin
	r3 = 0.0:0.01:5.0
	
	πmp3 = - (r̄3 ./ λ3) .+  r3 ./ λ3 
	
	trace4_0 = scatter(; x = r3, y = πmp3, mode = "lines", line_color = "Brown", 
		line_width = "3", name  = "MP")
	
	trace4_1 = scatter(; x = [3.0], y = [2.0], text =["1"], 
		textposition = "top center", name ="Eq1", mode="markers+text", 
		marker_size = "11",	marker_color = "Brown",
		textfont = attr(family="sans serif", size=18, color="black") )
	
	layout4_1 = Layout(;
		title_text ="Fig 4. MP Function",
		title_x = 0.5,
		hovermode = "x",
        xaxis = attr(title=" Real interest rate (r)", showgrid=true, 
			zeroline=false),
        xaxis_range = [2.0, 4.5],
		xaxis_tickvals = [2, 2.5, 3, 3.5, 4, 4.5],
        yaxis = attr(title="Rate of inflation (π)", zeroline = false),
		yaxis_range = [-0 , 5])

   p4_1 = Plot([trace4_0, trace4_1], layout4_1)
end;

# ╔═╡ 5eb27597-8fcf-4f35-8a49-818bdbce8951
begin
	p4_2 =[p4_1 p10_23]
end

# ╔═╡ 1385bba0-7daf-4fe6-a9f8-9cc2fa29e600
begin
	Y_ZL3 = Ā3 * m3 - (m3 *  ϕ3 * r̄3) / (1+λ3)
	πd_min3 = - Ā3 / ϕ3
end;

# ╔═╡ 9b5ac60d-6732-4636-a2b2-08ab6f7297ad
begin
	Y3_ZL = 11 : 0.01 : Y_ZL3

	πd3_normal = πd_max3 .- (1 ./(m3 .* ϕ3 .* λ3)) .* Y3_ZL
	
	πd3_ZL = πd_min3 .+ (1 ./(m3 .* ϕ3)) .* Y3_ZL

	trace10_24 = scatter(; x = Y3_ZL , y = πd3_normal , mode = "line", name ="AD", line_color = "Blue")
	
	trace10_25 = scatter(; x = Y3_ZL , y = πd3_ZL , mode = "line", name ="ADzl", line_color = "BlueViolet")


	trace10_26 = scatter(; x = [12.9, 15.1] , y = [π_ZL3 , π_ZL3] , line_dash="dashdot", 
		line_width = "0.5", name = "ZLB", line_color = "black")

		
	layout10_26 = Layout(
					title_text = "Fig 7. The AD/AS functions with a ZLB",
					title_x = 0.5,
					hovermode = "x",
					xaxis_title = "GDP trillion dollars (Y)",
					xaxis_range = [13 , 15],
					yaxis_range=[-6 ,8],
					yaxis_title = "Rate of inflation (π)",
					#yaxis_range=[-1 ,5], 
					)

	p10_27 = Plot([trace10_24, trace10_25, trace10_26], layout10_26)
	p10_28 = addtraces(p10_27, trace10_20, trace10_22, trace10_23) # trace2 is the AS function
end;

# ╔═╡ 3bc3f9b4-c716-4655-a3cd-e83efbdbf4b9
begin
	πs_min6 = πᵉ6 - γ3 * Yᴾ3 + ρ6
	πs6 = πs_min6 .+ γ3 .*Y3 
	
	trace10_61 = scatter(; x = Y3, y = πs6, mode="lines" , line_color = "Red", 
		line_width = "3", name  = "AS")
	
	trace10_62 = scatter(; x = [13.5], y = [-4.25], text =["2zl"], textposition = "top center", 
		name = "Eq2zl", mode = "markers+text", marker_size = "11", marker_color = "Blue",
		textfont = attr(family="sans serif", size=16, color="black"))
	
	trace10_63 = scatter(; x = [14.4211], y = [-0.105263], text =["2"], textposition = "top center", 
		name = "Eq2", mode = "markers+text", marker_size = "11", marker_color = "Blue",
		textfont = attr(family="sans serif", size=16, color="black"))
	
	trace10_64 = scatter(; x = [14.0], y = [-3.0], text =["3"], textposition = "bottom center", 
		name = "Eq3", mode = "markers+text", marker_size = "11", marker_color = "Blue",
		textfont = attr(family="sans serif", size=18, color="black"))
	
	
	layout10_61 = Layout(
					title_text = "The AD/AS curves with a ZLB",
					title_x = 0.5,
					hovermode = "x",
					xaxis_title = "GDP trillion dollars (Y)",
					xaxis_range = [13 , 15],
					yaxis_range=[-7 , 8],
					yaxis_title = "Rate of inflation (% points)",
					#yaxis_range=[-1 ,5], 
					)
	
	p10_61 = Plot([trace10_61], layout10_61)
	p10_62 = addtraces(p10_61, trace10_22, trace10_24, trace10_25, trace10_26, trace10_20, trace10_23, 
		trace10_63, trace10_62, trace10_64) 
	#	trace10_23, trace10_53, trace10_56) 
end

# ╔═╡ ecd1d9df-bec2-4d67-850b-1c263bfe2606
begin
	
	r3_ZL = -π_ZL3 : 0.001 : 5

	# Defining the two branches 

	πmp3_normal = -(r̄3 ./ λ3) .+  r3_ZL ./ λ3

	πmp3_ZL = -r3_ZL


	# Defining each of the functions we want to plot in each trace1

	trace10_30 = scatter(; x = r3_ZL , y = πmp3_normal , mode = "line", name ="MP", line_color = "Brown")
	
	trace10_31 = scatter(; x = r3_ZL , y = πmp3_ZL , mode = "line", name ="MPzl", 
		line_color = "Chocolate")

	trace10_32 = scatter(; x = [0.9 , 5.1] , y = [π_ZL3 , π_ZL3] , line_dash="dashdot", 
		line_width = "0.5", name = "ZLB", line_color = "black")


	trace10_33 = scatter(; x = [3], y = [2], text =["1"], textposition = "top center", 
					   name ="Eq1", mode="markers+text", marker_size= "11", marker_color="Brown",
					   textfont = attr(family="sans serif", size=18, color="black"))


	# The layout chosen

	layout10_33 = Layout(
					title_text = "Fig 6. The MP function with a ZLB",
					title_x = 0.5,
					hovermode = "x",
					xaxis_title = "Real interest rate (% points)",
					xaxis_range = [1, 5],
					yaxis_range=[-6 , 8],
					yaxis_title = "Rate of inflation (π)",
					#yaxis_range=[-1 ,5], 
					)

	# Plotting

	p10_33 = Plot([trace10_30, trace10_31, trace10_32, trace10_33], layout10_33)
end;

# ╔═╡ b2071081-ad56-43a8-8bf4-54802bfbc739
begin
	p10_34 =[p10_33  p10_28]
end

# ╔═╡ 1684b665-eae3-4e96-9f10-5909e09d9964
md"""
##### Exercise 5. Supporting cells
"""

# ╔═╡ 2a8b4884-a1e7-4803-a2b1-6ce85955de93
begin	
	ΔĀ4 = -0.2 															# the shock to the AD function
	Y_ZL3_ΔAbar4 = (Ā3 + ΔĀ4) * m3 - (m3 *  ϕ3 * r̄3) / (1+λ3)
	πd_min4 = - (Ā3 + ΔĀ4 ) / ϕ3
	πd_max4 = (Ā3 + ΔĀ4)  / (λ3 * ϕ3) - r̄3/λ3 							# The AD curve y-axis' intercept	
end;

# ╔═╡ dad66466-0f3f-4908-92a4-50c5b20e5be5
begin
	function my_model10_4!(F, v)
	
           	π = v[1]
           	Y = v[2]
		   	r = v[3]
			i = v[4]
				
           	F[1] = π + (1/(m3 * ϕ3 * λ3))*Y - πd_max4     	# AD curve = 0
           	F[2] = π - γ3 * Y - πs_min3                   	# AS curve = 0
			F[3] = λ3 * π - r + r̄3                     		# MP curve = 0 
			F[4] = i - π - r                               	# Fisher Equation = 0
	end
end

# ╔═╡ 6ac7c37a-96a5-49d8-8fff-b76e381b6367
begin
	solution10_4 = nlsolve(my_model10_4!, [0.0; 0.0; 0.0; 0.0])
	solution10_4.zero
end

# ╔═╡ f5fde724-21f9-4184-a317-8dfe75ec84ba
begin
	Y4_ZL = 12 : 0.0001 : Y_ZL3_ΔAbar4

	πd4_normal = πd_max4 .- (1 ./(m3 .* ϕ3 .* λ3)) .* Y4_ZL
	
	πd4_ZL = πd_min4 .+ (1 ./(m3 .* ϕ3)) .* Y4_ZL

	trace10_35 = scatter(; x = Y4_ZL , y = πd4_normal , mode = "line", name ="AD", line_color = "Blue")
	
	trace10_36 = scatter(; x = Y4_ZL , y = πd4_ZL  , mode = "line", name ="ADzl", line_color = "BlueViolet")
	
	trace10_37 = scatter(; x = [12.9, 15.1] , y = [π_ZL3 , π_ZL3] , line_dash="dashdot", 
		line_width = "0.5", name = "ZLB", line_color = "black")
	
	trace10_38 = scatter(; x = [13.7895], y = [1.05263], text =["2"], textposition = "top center", 
		name = "Eq2", mode = "markers+text", marker_size = "11", marker_color = "Blue",
		textfont = attr(family="sans serif", size=16, color="black"))
		
	layout10_38 = Layout(
					title_text = "Fig 9. The AD/AS functions with a ZLB",
					title_x = 0.5,
					hovermode = "x",
					xaxis_title = "GDP trillion dollars (Y)",
					xaxis_range = [13.25 , 14.5],
					yaxis_range=[-4 ,4],
					yaxis_title = "Rate of inflation (% points)",
					#yaxis_range=[-1 ,5], 
					)

	p10_36 = Plot([trace10_35, trace10_36, trace10_37], layout10_38)
	p10_37 = addtraces(p10_36, trace10_20, trace10_22, trace10_23, trace10_38) # trace2 is the AS function
end;

# ╔═╡ e00c9d65-074e-4cb3-8e08-b32a58ee92c4
begin
	
	trace10_39 = scatter(; x = [2.52631], y = [1.05263], text =["2"], textposition = "top center", 
					   name ="Eq2", mode="markers+text", marker_size= "11", marker_color="Brown",
					   textfont = attr(family="sans serif", size=16, color="black"))
	
	layout10_39 = Layout(
					title_text = "Fig 8. The MP function with a ZLB",
					title_x = 0.5,
					hovermode = "x",
					xaxis_title = "Real interest rate (% points)",
					xaxis_range = [1, 4],
					yaxis_range=[-4 , 4],
					yaxis_title = "Rate of inflation (% points)",
					#yaxis_range=[-1 ,5], 
					)

	p10_39 = Plot([trace10_39 , trace10_33], layout10_39)
	p10_40 = addtraces(p10_39, trace10_30, trace10_31, trace10_32)
end;

# ╔═╡ 495cbcdb-4120-4200-bfa6-5d1ad59308bb
begin
	p10_41 =[p10_40  p10_37]
end

# ╔═╡ 1e98e928-68bd-4102-bdc8-2536b4615eaa
begin

	trace10_42 = scatter(; x = Y3, y = πs3 .+ self_correction4, mode="lines" , 
		line_color = "DarkRed", line_width = "3", name  = "AS2")
	
	trace10_43 = scatter(; x = [14.0], y = [0.0], text =["3"], textposition = "top center", 
		name = "Eq3", mode = "markers+text", marker_size = "12", marker_color = "Blue",
		textfont = attr(family="sans serif", size=18, color="black"))
	

	p10_43 = Plot([trace10_20, trace10_22, trace10_42], layout10_38)
	p10_44 = addtraces(p10_43, trace10_35, trace10_36, trace10_37,  
		trace10_23, trace10_38, trace10_43) # trace10_20 is the YP function
end;

# ╔═╡ 1d7292a7-0dc6-4c0a-8e3c-fce761f3c499
begin
	
	trace10_45 = scatter(; x = [2], y = [0], text =["3"], textposition = "top center", 
					   name ="Eq3", mode="markers+text", marker_size= "10", marker_color="Brown",
					   textfont = attr(family="sans serif", size=16, color="black"))
	
	p10_45 = Plot([trace10_33, trace10_39, trace10_45], layout10_39)
	p10_46 = addtraces(p10_45, trace10_30, trace10_31, trace10_32)
end;

# ╔═╡ 5499984f-c56d-4a84-aa6b-24d48ec3b786
begin
	p10_47 =[p10_46  p10_44]
end

# ╔═╡ eeeb30c0-6b7d-42ff-93b8-dd647d72babe
md"""
##### Exercise 6. Supporting cells
"""

# ╔═╡ 91fd4da1-abe0-4096-adcd-581d41fbfc33
begin	
	Y_ZL3_ΔĀ5 = (Ā3 + ΔĀ5) * m3 - (m3 *  ϕ3 * r̄3) / (1+λ3)
	πd_min5 = - (Ā3 + ΔĀ5 ) / ϕ3
	πd_max5 = (Ā3 + ΔĀ5)  / (λ3 * ϕ3) - r̄3/λ3 # The AD curve y-axis' intercept	
end;

# ╔═╡ d4c73d36-fc14-4968-9166-660e05cd2078
begin
	Y5_ZL = 11 : 0.0001 : Y_ZL3_ΔĀ5

	πd5_normal = πd_max5 .- (1 ./(m3 .* ϕ3 .* λ3)) .* Y5_ZL
	
	πd5_ZL = πd_min5 .+ (1 ./(m3 .* ϕ3)) .* Y5_ZL

	trace10_50 = scatter(; x = Y5_ZL , y = πd5_normal , mode = "line", name ="AD", line_color = "Blue")
	
	trace10_51 = scatter(; x = Y5_ZL , y = πd5_ZL , mode = "line", name ="ADzl", line_color = "BlueViolet")
	
	trace10_52 = scatter(; x = [10.9, 15.1] , y = [π_ZL3 , π_ZL3] , line_dash="dashdot", 
		line_width = "0.5", name = "ZLB", line_color = "black")
	
	trace10_53 = scatter(; x = [13.4737], y = [-0.368421], text =["2"], textposition = "top center", 
		name = "Eq2", mode = "markers+text", marker_size = "11", marker_color = "Blue",
		textfont = attr(family="sans serif", size=16, color="black"))
		
	layout10_53 = Layout(
					title_text = "The AD/AS curves with a ZLB",
					title_x = 0.5,
					xaxis_title = "GDP trillion dollars (Y)",
					xaxis_range = [13 , 14.2],
					yaxis_range=[-3 , 3],
					yaxis_title = "Rate of inflation (% points)",
					#yaxis_range=[-1 ,5], 
					)

	p10_53 = Plot([trace10_50, trace10_51, trace10_52], layout10_53)
	p10_54 = addtraces(p10_53, trace10_20, trace10_22, trace10_23, trace10_53)
end

# ╔═╡ a81ad800-8dcc-4972-8245-5edf568aa611
self_correction5 = π_ZL3 - πs_min3 - γ3 * 13.666677

# ╔═╡ 291adf2b-f5d7-40da-9778-f2218045273d
begin
	trace10_55 = scatter(; x = Y3, y = πs3 .+ self_correction5, mode="lines" , 
		line_color = "DarkRed", line_width = "3", name  = "AS2")
	
	trace10_56 = scatter(; x = [13.66667], y = [π_ZL3], text =["3"], textposition = "top center", 
		name = "Eq3", mode = "markers+text", marker_size = "11", marker_color = "Blue",
		textfont = attr(family="sans serif", size=18, color="black"))
	
	p10_56 = Plot([trace10_55], layout10_53)
	p10_57 = addtraces(p10_56, trace10_20, trace10_50, trace10_51, trace10_52, trace10_22, 
		trace10_23, trace10_53, trace10_56) 
end

# ╔═╡ a2730369-1206-453d-930a-b6c779395cd5
begin
	function my_model10_5!(F, v)
	
           	π = v[1]
           	Y = v[2]
		   	r = v[3]
			i = v[4]
				
           	F[1] = π + (1/(m3 * ϕ3 * λ3))*Y - πd_max5     # AD function = 0
           	F[2] = π - γ3 * Y - πs_min3                   # AS function = 0
			F[3] = λ3 * π - r + r̄3                        # MP function = 0 
			F[4] = i - π - r                              # Fisher Equation = 0
	end
end

# ╔═╡ 56bf8ed2-5786-4e51-b9c8-7a68a0690283
begin
	solution10_5 = nlsolve(my_model10_5!, [0.0; 0.0; 0.0; 0.0])
	solution10_5.zero
end

# ╔═╡ e1c5c854-02c0-439c-9ba6-3f03be2a3d61
md"""
The new equilibrium at point 2 will be characterized by:

$$Y^{P} > Y_2 =13.4737 \ , \ \pi_2=-0.368421 \% \ , \ r_2=1.81579 \% \ , \ i_2=1.44737 \%.$$
"""

# ╔═╡ f9d1d025-6b70-4a54-bc8a-7f7c5cad318a
md"""
###### Solution of point 3 using NLsolve
"""

# ╔═╡ 13f44f40-72d2-493f-a584-e17b6d0e102b
md"""
Notice that at the kink of the ZLB, the AD curve consists of just one point:
$$\pi = \pi_{ZL3}.$$
Therefore, in the NSolve system the function F[1] should be written as: F[1] = π - π_ZL3
"""

# ╔═╡ f6d599fb-ac27-4c44-972f-28f300f43cdf
begin
	function my_model10_5a!(F, v)
	
           	π = v[1]
           	Y = v[2]
		   	r = v[3]
			i = v[4]				
           	
           	F[1] = π - π_ZL3 										# AD function = 0
			F[2] = π - γ3 * Y - (πs_min3 + self_correction5)        # AS function = 0
			F[3] = λ3 * π - r + r̄3                     				# MP function = 0 
			F[4] = i - π - r                               			# Fisher Equation = 0
	end
end

# ╔═╡ ed9a0b2a-68fb-4996-81b2-60ba59248207
begin
	solution10_5a = nlsolve(my_model10_5a!, [0.0; 0.0; 0.0; 0.0 ])
	solution10_5a.zero
end

# ╔═╡ b1ea211b-4ab8-49bd-96b7-dd8631cb689a
md"""
The new equilibrium under secular stagnation will be characterized by:

$$Y^{P} > Y_{3} =13.6667 \ , \ \pi_{3}=-1.33333 \% \ , \ r_{3}=1.33333 \% \ , \ i_{3}=0.0 \%.$$
"""

# ╔═╡ 47ac77f3-4f6d-4469-a2b7-edf35a049ed3
md"""
##### Exercise 7. Supporting cells
"""

# ╔═╡ 51d0366d-3224-4a48-9e8e-7c8e4d41fd7e
md"""
###### Solution of point 2 using NLsolve
"""

# ╔═╡ 3a2b7a9b-0f24-4166-ba46-fc0b3df74f46
begin
	function my_model10_6!(F, v)
	
           	π = v[1]
           	Y = v[2]
		   	r = v[3]
			i = v[4]
				
           	F[1] = π + (1/(m3 * ϕ3 * λ3))*Y - πd_max3     	# AD curve = 0
           	F[2] = π - γ3 * Y - πs_min6                   	# AS curve = 0
			F[3] = λ3 * π - r + r̄3                     		# MP curve = 0 
			F[4] = i - π - r                              	# Fisher Equation = 0
	end

	solution10_6 = nlsolve(my_model10_6!, [0.0; 0.0; 0.0; 0.0])
	solution10_6.zero
end

# ╔═╡ 9b68bbc2-d7f3-4bd7-9993-e4fe20363645


# ╔═╡ 5a8e44d3-ba71-4bb7-b6cc-509b08c9e1bb
md"""
The new equilibrium in point 2 will be characterized by:

$$Y^{P} < Y_2 =14.4211 \ , \ \pi_2=-0.105263 \% \ , \ r_2=1.94737 \% \ , \ i_2=1.84211 \%.$$
"""

# ╔═╡ 1b076f8f-f69f-40af-afe5-68749a409785
md"""
###### Solution of point 2zl using NLsolve
"""

# ╔═╡ 1dfbe74a-61d7-48d8-823b-4d19efb3e6be
md"""
Notice that the AD function in the ZLB is given by the expression:

$$Y=m \cdot \bar{A}+m \cdot \phi \cdot \pi_{Z L}$$
So the F[1] has to be rewritten in the NSolve code as: 
$\mathrm{F}[1]=\mathrm{Y}-\mathrm{m} 3 * \overline{A}3-(\mathrm{m} 3 * \phi 3) * \pi$

"""

# ╔═╡ 960c426d-e69d-46f4-bc7c-a9c921b1a257
begin
	function my_model10_6a!(F, v)
	
           	π = v[1]
           	Y = v[2]
		   	r = v[3]
			i = v[4]
				
           	F[1] = Y - m3 * Ā3 - (m3 * ϕ3)* π     	  	# AD function = 0
           	F[2] = π - γ3 * Y - πs_min6                 # AS function = 0
			F[3] = λ3 * π - r + r̄3                      # MP function = 0 
			F[4] = i - π - r                            # Fisher Equation = 0
	end
	
	solution10_6a = nlsolve(my_model10_6a!, [0.0; 0.0; 0.0; 0.0])
	solution10_6a.zero
end

# ╔═╡ 29a9d34f-f239-43eb-9ce8-9c07c35d2d4e


# ╔═╡ 81536d10-9345-4889-b6d1-8fda40d2b0cb
md"""
The new equilibrium in point 2 will be characterized by:

$$Y^{P} > Y_{2zl} =13.5 \ , \ \pi_{2zl}=-4.25 \% \ , \ r_{2zl}=-0.125 \% \ , \ i_{2zl}=-4.375 \%.$$
"""

# ╔═╡ 00000000-0000-0000-0000-000000000001
PLUTO_PROJECT_TOML_CONTENTS = """
[deps]
CSV = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
Dates = "ade2ca70-3891-5945-98fb-dc099432e06a"
HypertextLiteral = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
NLsolve = "2774e3e8-f4cf-5e23-947b-6d7e65073b56"
PeriodicalDates = "276e7ca9-e0d7-440b-97bc-a6ae82f545b1"
PlotlyBase = "a03496cd-edff-5a9b-9e67-9cda94a718b5"
PlutoPlotly = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
PlutoUI = "7f904dfe-b85e-4ff6-b463-dae2292396a8"

[compat]
CSV = "~0.10.7"
DataFrames = "~1.4.2"
HypertextLiteral = "~0.9.4"
NLsolve = "~4.5.1"
PeriodicalDates = "~2.0.0"
PlotlyBase = "~0.8.19"
PlutoPlotly = "~0.3.6"
PlutoUI = "~0.7.48"
"""

# ╔═╡ 00000000-0000-0000-0000-000000000002
PLUTO_MANIFEST_TOML_CONTENTS = """
# This file is machine-generated - editing it directly is not advised

julia_version = "1.8.5"
manifest_format = "2.0"
project_hash = "079bd01d8780c41c0ca0bd9cdb536d1e4b0f22f1"

[[deps.AbstractPlutoDingetjes]]
deps = ["Pkg"]
git-tree-sha1 = "8eaf9f1b4921132a4cff3f36a1d9ba923b14a481"
uuid = "6e696c72-6542-2067-7265-42206c756150"
version = "1.1.4"

[[deps.ArgTools]]
uuid = "0dad84c5-d112-42e6-8d28-ef12dabb789f"
version = "1.1.1"

[[deps.ArrayInterfaceCore]]
deps = ["LinearAlgebra", "SparseArrays", "SuiteSparse"]
git-tree-sha1 = "14c3f84a763848906ac681f94cf469a851601d92"
uuid = "30b0a656-2188-435a-8636-2ec0e6a096e2"
version = "0.1.28"

[[deps.Artifacts]]
uuid = "56f22d72-fd6d-98f1-02f0-08ddc0907c33"

[[deps.Base64]]
uuid = "2a0f44e3-6c83-55bd-87e4-b1978d98bd5f"

[[deps.CSV]]
deps = ["CodecZlib", "Dates", "FilePathsBase", "InlineStrings", "Mmap", "Parsers", "PooledArrays", "SentinelArrays", "SnoopPrecompile", "Tables", "Unicode", "WeakRefStrings", "WorkerUtilities"]
git-tree-sha1 = "c700cce799b51c9045473de751e9319bdd1c6e94"
uuid = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
version = "0.10.9"

[[deps.ChainRulesCore]]
deps = ["Compat", "LinearAlgebra", "SparseArrays"]
git-tree-sha1 = "e7ff6cadf743c098e08fca25c91103ee4303c9bb"
uuid = "d360d2e6-b24c-11e9-a2a3-2a2ae2dbcce4"
version = "1.15.6"

[[deps.ChangesOfVariables]]
deps = ["ChainRulesCore", "LinearAlgebra", "Test"]
git-tree-sha1 = "38f7a08f19d8810338d4f5085211c7dfa5d5bdd8"
uuid = "9e997f8a-9a97-42d5-a9f1-ce6bfc15e2c0"
version = "0.1.4"

[[deps.CodecZlib]]
deps = ["TranscodingStreams", "Zlib_jll"]
git-tree-sha1 = "ded953804d019afa9a3f98981d99b33e3db7b6da"
uuid = "944b1d66-785c-5afd-91f1-9de20f533193"
version = "0.7.0"

[[deps.ColorSchemes]]
deps = ["ColorTypes", "ColorVectorSpace", "Colors", "FixedPointNumbers", "Random", "SnoopPrecompile"]
git-tree-sha1 = "aa3edc8f8dea6cbfa176ee12f7c2fc82f0608ed3"
uuid = "35d6a980-a343-548e-a6ea-1d62b119f2f4"
version = "3.20.0"

[[deps.ColorTypes]]
deps = ["FixedPointNumbers", "Random"]
git-tree-sha1 = "eb7f0f8307f71fac7c606984ea5fb2817275d6e4"
uuid = "3da002f7-5984-5a60-b8a6-cbb66c0b333f"
version = "0.11.4"

[[deps.ColorVectorSpace]]
deps = ["ColorTypes", "FixedPointNumbers", "LinearAlgebra", "SpecialFunctions", "Statistics", "TensorCore"]
git-tree-sha1 = "600cc5508d66b78aae350f7accdb58763ac18589"
uuid = "c3611d14-8923-5661-9e6a-0046d554d3a4"
version = "0.9.10"

[[deps.Colors]]
deps = ["ColorTypes", "FixedPointNumbers", "Reexport"]
git-tree-sha1 = "fc08e5930ee9a4e03f84bfb5211cb54e7769758a"
uuid = "5ae59095-9a9b-59fe-a467-6f913c188581"
version = "0.12.10"

[[deps.CommonSubexpressions]]
deps = ["MacroTools", "Test"]
git-tree-sha1 = "7b8a93dba8af7e3b42fecabf646260105ac373f7"
uuid = "bbf7d656-a473-5ed7-a52c-81e309532950"
version = "0.3.0"

[[deps.Compat]]
deps = ["Dates", "LinearAlgebra", "UUIDs"]
git-tree-sha1 = "00a2cccc7f098ff3b66806862d275ca3db9e6e5a"
uuid = "34da2185-b29b-5c13-b0c7-acf172513d20"
version = "4.5.0"

[[deps.CompilerSupportLibraries_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "e66e0078-7015-5450-92f7-15fbd957f2ae"
version = "1.0.1+0"

[[deps.ConstructionBase]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "fb21ddd70a051d882a1686a5a550990bbe371a95"
uuid = "187b0558-2788-49d3-abe0-74a17ed4e7c9"
version = "1.4.1"

[[deps.Crayons]]
git-tree-sha1 = "249fe38abf76d48563e2f4556bebd215aa317e15"
uuid = "a8cc5b0e-0ffa-5ad4-8c14-923d3ee1735f"
version = "4.1.1"

[[deps.DataAPI]]
git-tree-sha1 = "e8119c1a33d267e16108be441a287a6981ba1630"
uuid = "9a962f9c-6df0-11e9-0e5d-c546b8b5ee8a"
version = "1.14.0"

[[deps.DataFrames]]
deps = ["Compat", "DataAPI", "Future", "InvertedIndices", "IteratorInterfaceExtensions", "LinearAlgebra", "Markdown", "Missings", "PooledArrays", "PrettyTables", "Printf", "REPL", "Random", "Reexport", "SnoopPrecompile", "SortingAlgorithms", "Statistics", "TableTraits", "Tables", "Unicode"]
git-tree-sha1 = "d4f69885afa5e6149d0cab3818491565cf41446d"
uuid = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
version = "1.4.4"

[[deps.DataStructures]]
deps = ["Compat", "InteractiveUtils", "OrderedCollections"]
git-tree-sha1 = "d1fff3a548102f48987a52a2e0d114fa97d730f0"
uuid = "864edb3b-99cc-5e75-8d2d-829cb0a9cfe8"
version = "0.18.13"

[[deps.DataValueInterfaces]]
git-tree-sha1 = "bfc1187b79289637fa0ef6d4436ebdfe6905cbd6"
uuid = "e2d170a0-9d28-54be-80f0-106bbe20a464"
version = "1.0.0"

[[deps.Dates]]
deps = ["Printf"]
uuid = "ade2ca70-3891-5945-98fb-dc099432e06a"

[[deps.DelimitedFiles]]
deps = ["Mmap"]
uuid = "8bb1440f-4735-579b-a4ab-409b98df4dab"

[[deps.DiffResults]]
deps = ["StaticArraysCore"]
git-tree-sha1 = "782dd5f4561f5d267313f23853baaaa4c52ea621"
uuid = "163ba53b-c6d8-5494-b064-1a9d43ac40c5"
version = "1.1.0"

[[deps.DiffRules]]
deps = ["IrrationalConstants", "LogExpFunctions", "NaNMath", "Random", "SpecialFunctions"]
git-tree-sha1 = "c5b6685d53f933c11404a3ae9822afe30d522494"
uuid = "b552c78f-8df3-52c6-915a-8e097449b14b"
version = "1.12.2"

[[deps.Distances]]
deps = ["LinearAlgebra", "SparseArrays", "Statistics", "StatsAPI"]
git-tree-sha1 = "3258d0659f812acde79e8a74b11f17ac06d0ca04"
uuid = "b4f34e82-e78d-54a5-968a-f98e89d6e8f7"
version = "0.10.7"

[[deps.Distributed]]
deps = ["Random", "Serialization", "Sockets"]
uuid = "8ba89e20-285c-5b6f-9357-94700520ee1b"

[[deps.DocStringExtensions]]
deps = ["LibGit2"]
git-tree-sha1 = "2fb1e02f2b635d0845df5d7c167fec4dd739b00d"
uuid = "ffbed154-4ef7-542d-bbb7-c09d3a79fcae"
version = "0.9.3"

[[deps.Downloads]]
deps = ["ArgTools", "FileWatching", "LibCURL", "NetworkOptions"]
uuid = "f43a241f-c20a-4ad4-852c-f6b1247861c6"
version = "1.6.0"

[[deps.FilePathsBase]]
deps = ["Compat", "Dates", "Mmap", "Printf", "Test", "UUIDs"]
git-tree-sha1 = "e27c4ebe80e8699540f2d6c805cc12203b614f12"
uuid = "48062228-2e41-5def-b9a4-89aafe57970f"
version = "0.9.20"

[[deps.FileWatching]]
uuid = "7b1f6079-737a-58dc-b8bc-7a2ca5c1b5ee"

[[deps.FiniteDiff]]
deps = ["ArrayInterfaceCore", "LinearAlgebra", "Requires", "Setfield", "SparseArrays", "StaticArrays"]
git-tree-sha1 = "04ed1f0029b6b3af88343e439b995141cb0d0b8d"
uuid = "6a86dc24-6348-571c-b903-95158fe2bd41"
version = "2.17.0"

[[deps.FixedPointNumbers]]
deps = ["Statistics"]
git-tree-sha1 = "335bfdceacc84c5cdf16aadc768aa5ddfc5383cc"
uuid = "53c48c17-4a7d-5ca2-90c5-79b7896eea93"
version = "0.8.4"

[[deps.Formatting]]
deps = ["Printf"]
git-tree-sha1 = "8339d61043228fdd3eb658d86c926cb282ae72a8"
uuid = "59287772-0a20-5a39-b81b-1366585eb4c0"
version = "0.4.2"

[[deps.ForwardDiff]]
deps = ["CommonSubexpressions", "DiffResults", "DiffRules", "LinearAlgebra", "LogExpFunctions", "NaNMath", "Preferences", "Printf", "Random", "SpecialFunctions", "StaticArrays"]
git-tree-sha1 = "a69dd6db8a809f78846ff259298678f0d6212180"
uuid = "f6369f11-7733-5829-9624-2563aa707210"
version = "0.10.34"

[[deps.Future]]
deps = ["Random"]
uuid = "9fa8497b-333b-5362-9e8d-4d0656e87820"

[[deps.Hyperscript]]
deps = ["Test"]
git-tree-sha1 = "8d511d5b81240fc8e6802386302675bdf47737b9"
uuid = "47d2ed2b-36de-50cf-bf87-49c2cf4b8b91"
version = "0.0.4"

[[deps.HypertextLiteral]]
deps = ["Tricks"]
git-tree-sha1 = "c47c5fa4c5308f27ccaac35504858d8914e102f9"
uuid = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
version = "0.9.4"

[[deps.IOCapture]]
deps = ["Logging", "Random"]
git-tree-sha1 = "f7be53659ab06ddc986428d3a9dcc95f6fa6705a"
uuid = "b5f81e59-6552-4d32-b1f0-c071b021bf89"
version = "0.2.2"

[[deps.InlineStrings]]
deps = ["Parsers"]
git-tree-sha1 = "0cf92ec945125946352f3d46c96976ab972bde6f"
uuid = "842dd82b-1e85-43dc-bf29-5d0ee9dffc48"
version = "1.3.2"

[[deps.InteractiveUtils]]
deps = ["Markdown"]
uuid = "b77e0a4c-d291-57a0-90e8-8db25a27a240"

[[deps.InverseFunctions]]
deps = ["Test"]
git-tree-sha1 = "49510dfcb407e572524ba94aeae2fced1f3feb0f"
uuid = "3587e190-3f89-42d0-90ee-14403ec27112"
version = "0.1.8"

[[deps.InvertedIndices]]
git-tree-sha1 = "82aec7a3dd64f4d9584659dc0b62ef7db2ef3e19"
uuid = "41ab1584-1d38-5bbf-9106-f11c6c58b48f"
version = "1.2.0"

[[deps.IrrationalConstants]]
git-tree-sha1 = "7fd44fd4ff43fc60815f8e764c0f352b83c49151"
uuid = "92d709cd-6900-40b7-9082-c6be49f344b6"
version = "0.1.1"

[[deps.IteratorInterfaceExtensions]]
git-tree-sha1 = "a3f24677c21f5bbe9d2a714f95dcd58337fb2856"
uuid = "82899510-4779-5014-852e-03e436cf321d"
version = "1.0.0"

[[deps.JLLWrappers]]
deps = ["Preferences"]
git-tree-sha1 = "abc9885a7ca2052a736a600f7fa66209f96506e1"
uuid = "692b3bcd-3c85-4b1f-b108-f13ce0eb3210"
version = "1.4.1"

[[deps.JSON]]
deps = ["Dates", "Mmap", "Parsers", "Unicode"]
git-tree-sha1 = "3c837543ddb02250ef42f4738347454f95079d4e"
uuid = "682c06a0-de6a-54ab-a142-c8b1cf79cde6"
version = "0.21.3"

[[deps.LaTeXStrings]]
git-tree-sha1 = "f2355693d6778a178ade15952b7ac47a4ff97996"
uuid = "b964fa9f-0449-5b57-a5c2-d3ea65f4040f"
version = "1.3.0"

[[deps.LibCURL]]
deps = ["LibCURL_jll", "MozillaCACerts_jll"]
uuid = "b27032c2-a3e7-50c8-80cd-2d36dbcbfd21"
version = "0.6.3"

[[deps.LibCURL_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll", "Zlib_jll", "nghttp2_jll"]
uuid = "deac9b47-8bc7-5906-a0fe-35ac56dc84c0"
version = "7.84.0+0"

[[deps.LibGit2]]
deps = ["Base64", "NetworkOptions", "Printf", "SHA"]
uuid = "76f85450-5226-5b5a-8eaa-529ad045b433"

[[deps.LibSSH2_jll]]
deps = ["Artifacts", "Libdl", "MbedTLS_jll"]
uuid = "29816b5a-b9ab-546f-933c-edad1886dfa8"
version = "1.10.2+0"

[[deps.Libdl]]
uuid = "8f399da3-3557-5675-b5ff-fb832c97cbdb"

[[deps.LineSearches]]
deps = ["LinearAlgebra", "NLSolversBase", "NaNMath", "Parameters", "Printf"]
git-tree-sha1 = "7bbea35cec17305fc70a0e5b4641477dc0789d9d"
uuid = "d3d80556-e9d4-5f37-9878-2ab0fcc64255"
version = "7.2.0"

[[deps.LinearAlgebra]]
deps = ["Libdl", "libblastrampoline_jll"]
uuid = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"

[[deps.LogExpFunctions]]
deps = ["ChainRulesCore", "ChangesOfVariables", "DocStringExtensions", "InverseFunctions", "IrrationalConstants", "LinearAlgebra"]
git-tree-sha1 = "946607f84feb96220f480e0422d3484c49c00239"
uuid = "2ab3a3ac-af41-5b50-aa03-7779005ae688"
version = "0.3.19"

[[deps.Logging]]
uuid = "56ddb016-857b-54e1-b83d-db4d58db5568"

[[deps.MIMEs]]
git-tree-sha1 = "65f28ad4b594aebe22157d6fac869786a255b7eb"
uuid = "6c6e2e6c-3030-632d-7369-2d6c69616d65"
version = "0.1.4"

[[deps.MacroTools]]
deps = ["Markdown", "Random"]
git-tree-sha1 = "42324d08725e200c23d4dfb549e0d5d89dede2d2"
uuid = "1914dd2f-81c6-5fcd-8719-6d5c9610ff09"
version = "0.5.10"

[[deps.Markdown]]
deps = ["Base64"]
uuid = "d6f4376e-aef5-505a-96c1-9c027394607a"

[[deps.MbedTLS_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "c8ffd9c3-330d-5841-b78e-0817d7145fa1"
version = "2.28.0+0"

[[deps.Missings]]
deps = ["DataAPI"]
git-tree-sha1 = "f66bdc5de519e8f8ae43bdc598782d35a25b1272"
uuid = "e1d29d7a-bbdc-5cf2-9ac0-f12de2c33e28"
version = "1.1.0"

[[deps.Mmap]]
uuid = "a63ad114-7e13-5084-954f-fe012c677804"

[[deps.MozillaCACerts_jll]]
uuid = "14a3606d-f60d-562e-9121-12d972cd8159"
version = "2022.2.1"

[[deps.NLSolversBase]]
deps = ["DiffResults", "Distributed", "FiniteDiff", "ForwardDiff"]
git-tree-sha1 = "a0b464d183da839699f4c79e7606d9d186ec172c"
uuid = "d41bc354-129a-5804-8e4c-c37616107c6c"
version = "7.8.3"

[[deps.NLsolve]]
deps = ["Distances", "LineSearches", "LinearAlgebra", "NLSolversBase", "Printf", "Reexport"]
git-tree-sha1 = "019f12e9a1a7880459d0173c182e6a99365d7ac1"
uuid = "2774e3e8-f4cf-5e23-947b-6d7e65073b56"
version = "4.5.1"

[[deps.NaNMath]]
deps = ["OpenLibm_jll"]
git-tree-sha1 = "a7c3d1da1189a1c2fe843a3bfa04d18d20eb3211"
uuid = "77ba4419-2d1f-58cd-9bb1-8ffee604a2e3"
version = "1.0.1"

[[deps.NetworkOptions]]
uuid = "ca575930-c2e3-43a9-ace4-1e988b2c1908"
version = "1.2.0"

[[deps.OpenBLAS_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "Libdl"]
uuid = "4536629a-c528-5b80-bd46-f80d51c5b363"
version = "0.3.20+0"

[[deps.OpenLibm_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "05823500-19ac-5b8b-9628-191a04bc5112"
version = "0.8.1+0"

[[deps.OpenSpecFun_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "13652491f6856acfd2db29360e1bbcd4565d04f1"
uuid = "efe28fd5-8261-553b-a9e1-b2916fc3738e"
version = "0.5.5+0"

[[deps.OrderedCollections]]
git-tree-sha1 = "85f8e6578bf1f9ee0d11e7bb1b1456435479d47c"
uuid = "bac558e1-5e72-5ebc-8fee-abe8a469f55d"
version = "1.4.1"

[[deps.Parameters]]
deps = ["OrderedCollections", "UnPack"]
git-tree-sha1 = "34c0e9ad262e5f7fc75b10a9952ca7692cfc5fbe"
uuid = "d96e819e-fc66-5662-9728-84c9c7592b0a"
version = "0.12.3"

[[deps.Parsers]]
deps = ["Dates", "SnoopPrecompile"]
git-tree-sha1 = "6466e524967496866901a78fca3f2e9ea445a559"
uuid = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"
version = "2.5.2"

[[deps.PeriodicalDates]]
deps = ["Dates", "Printf", "RecipesBase"]
git-tree-sha1 = "e616941f8093e50a373e7d51875143213f82eca4"
uuid = "276e7ca9-e0d7-440b-97bc-a6ae82f545b1"
version = "2.0.0"

[[deps.Pkg]]
deps = ["Artifacts", "Dates", "Downloads", "LibGit2", "Libdl", "Logging", "Markdown", "Printf", "REPL", "Random", "SHA", "Serialization", "TOML", "Tar", "UUIDs", "p7zip_jll"]
uuid = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"
version = "1.8.0"

[[deps.PlotlyBase]]
deps = ["ColorSchemes", "Dates", "DelimitedFiles", "DocStringExtensions", "JSON", "LaTeXStrings", "Logging", "Parameters", "Pkg", "REPL", "Requires", "Statistics", "UUIDs"]
git-tree-sha1 = "56baf69781fc5e61607c3e46227ab17f7040ffa2"
uuid = "a03496cd-edff-5a9b-9e67-9cda94a718b5"
version = "0.8.19"

[[deps.PlutoPlotly]]
deps = ["AbstractPlutoDingetjes", "Colors", "Dates", "HypertextLiteral", "InteractiveUtils", "LaTeXStrings", "Markdown", "PlotlyBase", "PlutoUI", "Reexport"]
git-tree-sha1 = "dec81dcd52748ffc59ce3582e709414ff78d947f"
uuid = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
version = "0.3.6"

[[deps.PlutoUI]]
deps = ["AbstractPlutoDingetjes", "Base64", "ColorTypes", "Dates", "FixedPointNumbers", "Hyperscript", "HypertextLiteral", "IOCapture", "InteractiveUtils", "JSON", "Logging", "MIMEs", "Markdown", "Random", "Reexport", "URIs", "UUIDs"]
git-tree-sha1 = "eadad7b14cf046de6eb41f13c9275e5aa2711ab6"
uuid = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
version = "0.7.49"

[[deps.PooledArrays]]
deps = ["DataAPI", "Future"]
git-tree-sha1 = "a6062fe4063cdafe78f4a0a81cfffb89721b30e7"
uuid = "2dfb63ee-cc39-5dd5-95bd-886bf059d720"
version = "1.4.2"

[[deps.Preferences]]
deps = ["TOML"]
git-tree-sha1 = "47e5f437cc0e7ef2ce8406ce1e7e24d44915f88d"
uuid = "21216c6a-2e73-6563-6e65-726566657250"
version = "1.3.0"

[[deps.PrettyTables]]
deps = ["Crayons", "Formatting", "LaTeXStrings", "Markdown", "Reexport", "StringManipulation", "Tables"]
git-tree-sha1 = "96f6db03ab535bdb901300f88335257b0018689d"
uuid = "08abe8d2-0d0c-5749-adfa-8a2ac140af0d"
version = "2.2.2"

[[deps.Printf]]
deps = ["Unicode"]
uuid = "de0858da-6303-5e67-8744-51eddeeeb8d7"

[[deps.REPL]]
deps = ["InteractiveUtils", "Markdown", "Sockets", "Unicode"]
uuid = "3fa0cd96-eef1-5676-8a61-b3b8758bbffb"

[[deps.Random]]
deps = ["SHA", "Serialization"]
uuid = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"

[[deps.RecipesBase]]
deps = ["SnoopPrecompile"]
git-tree-sha1 = "261dddd3b862bd2c940cf6ca4d1c8fe593e457c8"
uuid = "3cdcf5f2-1ef4-517c-9805-6587b60abb01"
version = "1.3.3"

[[deps.Reexport]]
git-tree-sha1 = "45e428421666073eab6f2da5c9d310d99bb12f9b"
uuid = "189a3867-3050-52da-a836-e630ba90ab69"
version = "1.2.2"

[[deps.Requires]]
deps = ["UUIDs"]
git-tree-sha1 = "838a3a4188e2ded87a4f9f184b4b0d78a1e91cb7"
uuid = "ae029012-a4dd-5104-9daa-d747884805df"
version = "1.3.0"

[[deps.SHA]]
uuid = "ea8e919c-243c-51af-8825-aaa63cd721ce"
version = "0.7.0"

[[deps.SentinelArrays]]
deps = ["Dates", "Random"]
git-tree-sha1 = "efd23b378ea5f2db53a55ae53d3133de4e080aa9"
uuid = "91c51154-3ec4-41a3-a24f-3f23e20d615c"
version = "1.3.16"

[[deps.Serialization]]
uuid = "9e88b42a-f829-5b0c-bbe9-9e923198166b"

[[deps.Setfield]]
deps = ["ConstructionBase", "Future", "MacroTools", "StaticArraysCore"]
git-tree-sha1 = "e2cc6d8c88613c05e1defb55170bf5ff211fbeac"
uuid = "efcf1570-3423-57d1-acb7-fd33fddbac46"
version = "1.1.1"

[[deps.SnoopPrecompile]]
git-tree-sha1 = "f604441450a3c0569830946e5b33b78c928e1a85"
uuid = "66db9d55-30c0-4569-8b51-7e840670fc0c"
version = "1.0.1"

[[deps.Sockets]]
uuid = "6462fe0b-24de-5631-8697-dd941f90decc"

[[deps.SortingAlgorithms]]
deps = ["DataStructures"]
git-tree-sha1 = "a4ada03f999bd01b3a25dcaa30b2d929fe537e00"
uuid = "a2af1166-a08f-5f64-846c-94a0d3cef48c"
version = "1.1.0"

[[deps.SparseArrays]]
deps = ["LinearAlgebra", "Random"]
uuid = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"

[[deps.SpecialFunctions]]
deps = ["ChainRulesCore", "IrrationalConstants", "LogExpFunctions", "OpenLibm_jll", "OpenSpecFun_jll"]
git-tree-sha1 = "d75bda01f8c31ebb72df80a46c88b25d1c79c56d"
uuid = "276daf66-3868-5448-9aa4-cd146d93841b"
version = "2.1.7"

[[deps.StaticArrays]]
deps = ["LinearAlgebra", "Random", "StaticArraysCore", "Statistics"]
git-tree-sha1 = "6954a456979f23d05085727adb17c4551c19ecd1"
uuid = "90137ffa-7385-5640-81b9-e52037218182"
version = "1.5.12"

[[deps.StaticArraysCore]]
git-tree-sha1 = "6b7ba252635a5eff6a0b0664a41ee140a1c9e72a"
uuid = "1e83bf80-4336-4d27-bf5d-d5a4f845583c"
version = "1.4.0"

[[deps.Statistics]]
deps = ["LinearAlgebra", "SparseArrays"]
uuid = "10745b16-79ce-11e8-11f9-7d13ad32a3b2"

[[deps.StatsAPI]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "f9af7f195fb13589dd2e2d57fdb401717d2eb1f6"
uuid = "82ae8749-77ed-4fe6-ae5f-f523153014b0"
version = "1.5.0"

[[deps.StringManipulation]]
git-tree-sha1 = "46da2434b41f41ac3594ee9816ce5541c6096123"
uuid = "892a3eda-7b42-436c-8928-eab12a02cf0e"
version = "0.3.0"

[[deps.SuiteSparse]]
deps = ["Libdl", "LinearAlgebra", "Serialization", "SparseArrays"]
uuid = "4607b0f0-06f3-5cda-b6b1-a6196a1729e9"

[[deps.TOML]]
deps = ["Dates"]
uuid = "fa267f1f-6049-4f14-aa54-33bafae1ed76"
version = "1.0.0"

[[deps.TableTraits]]
deps = ["IteratorInterfaceExtensions"]
git-tree-sha1 = "c06b2f539df1c6efa794486abfb6ed2022561a39"
uuid = "3783bdb8-4a98-5b6b-af9a-565f29a5fe9c"
version = "1.0.1"

[[deps.Tables]]
deps = ["DataAPI", "DataValueInterfaces", "IteratorInterfaceExtensions", "LinearAlgebra", "OrderedCollections", "TableTraits", "Test"]
git-tree-sha1 = "c79322d36826aa2f4fd8ecfa96ddb47b174ac78d"
uuid = "bd369af6-aec1-5ad0-b16a-f7cc5008161c"
version = "1.10.0"

[[deps.Tar]]
deps = ["ArgTools", "SHA"]
uuid = "a4e569a6-e804-4fa4-b0f3-eef7a1d5b13e"
version = "1.10.1"

[[deps.TensorCore]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1feb45f88d133a655e001435632f019a9a1bcdb6"
uuid = "62fd8b95-f654-4bbd-a8a5-9c27f68ccd50"
version = "0.1.1"

[[deps.Test]]
deps = ["InteractiveUtils", "Logging", "Random", "Serialization"]
uuid = "8dfed614-e22c-5e08-85e1-65c5234f0b40"

[[deps.TranscodingStreams]]
deps = ["Random", "Test"]
git-tree-sha1 = "e4bdc63f5c6d62e80eb1c0043fcc0360d5950ff7"
uuid = "3bb67fe8-82b1-5028-8e26-92a6c54297fa"
version = "0.9.10"

[[deps.Tricks]]
git-tree-sha1 = "6bac775f2d42a611cdfcd1fb217ee719630c4175"
uuid = "410a4b4d-49e4-4fbc-ab6d-cb71b17b3775"
version = "0.1.6"

[[deps.URIs]]
git-tree-sha1 = "ac00576f90d8a259f2c9d823e91d1de3fd44d348"
uuid = "5c2747f8-b7ea-4ff2-ba2e-563bfd36b1d4"
version = "1.4.1"

[[deps.UUIDs]]
deps = ["Random", "SHA"]
uuid = "cf7118a7-6976-5b1a-9a39-7adc72f591a4"

[[deps.UnPack]]
git-tree-sha1 = "387c1f73762231e86e0c9c5443ce3b4a0a9a0c2b"
uuid = "3a884ed6-31ef-47d7-9d2a-63182c4928ed"
version = "1.0.2"

[[deps.Unicode]]
uuid = "4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5"

[[deps.WeakRefStrings]]
deps = ["DataAPI", "InlineStrings", "Parsers"]
git-tree-sha1 = "b1be2855ed9ed8eac54e5caff2afcdb442d52c23"
uuid = "ea10d353-3f73-51f8-a26c-33c1cb351aa5"
version = "1.4.2"

[[deps.WorkerUtilities]]
git-tree-sha1 = "cd1659ba0d57b71a464a29e64dbc67cfe83d54e7"
uuid = "76eceee3-57b5-4d4a-8e66-0e911cebbf60"
version = "1.6.1"

[[deps.Zlib_jll]]
deps = ["Libdl"]
uuid = "83775a58-1f1d-513f-b197-d71354ab007a"
version = "1.2.12+3"

[[deps.libblastrampoline_jll]]
deps = ["Artifacts", "Libdl", "OpenBLAS_jll"]
uuid = "8e850b90-86db-534c-a0d3-1478176c7d93"
version = "5.1.1+0"

[[deps.nghttp2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850ede-7688-5339-a07c-302acd2aaf8d"
version = "1.48.0+0"

[[deps.p7zip_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "3f19e933-33d8-53b3-aaab-bd5110c3b7a0"
version = "17.4.0+0"
"""

# ╔═╡ Cell order:
# ╟─943edb36-acc8-4b2a-94ab-c544f9eb279b
# ╟─1245aef9-18ce-42f5-af82-c2e6cce70798
# ╟─4618a663-f77e-4ccc-9b00-e84dbd5beb34
# ╠═a145e2fe-a903-11eb-160f-df7ea83fa3e6
# ╟─74de9f66-d5b4-4e0f-a75d-5177b1842191
# ╟─0bc2eb14-3b5a-4203-82dd-3ef2044a2c93
# ╟─7e025b57-5962-4ab8-85a0-a3cad37b1be5
# ╟─5999a598-0408-44f1-bd33-c4d0fa5a6006
# ╟─b6f5869c-62af-4e81-ae73-912fe522896e
# ╟─99fc937d-f798-418a-a57f-5ddc3feb5f3e
# ╠═9446db0c-db46-4d42-beca-55f152f0e8f5
# ╠═f6081970-7d0e-4aba-b712-bca0a06d4319
# ╠═8362fd70-ae19-45f8-bb6d-386946cd793c
# ╠═7d4589e5-37a5-4803-97bc-24c9a8ea61a5
# ╟─14a93d0a-2005-4406-9150-319d6583a518
# ╟─41c7afb6-a57b-45a7-8b9d-bb30902aab8b
# ╟─0836abe7-5ea6-4189-b783-a2215b6acd72
# ╟─4ce820d4-9eb2-4acd-92c7-820cf46d08ae
# ╠═66f39956-3665-4259-b10c-a82f3985b54b
# ╟─d0e70b80-1e84-412c-9e5f-0161a0b0a935
# ╠═57d7d8ed-dce5-4177-b92b-1114b538814d
# ╟─2207f310-b6e0-40ca-9d11-6bd90adbd997
# ╠═1dc9dce6-d373-4eb7-8e30-1030160accb4
# ╟─2275faad-ae60-49b6-a2e9-eda7a806b134
# ╟─b0679f0a-69fc-427d-aae5-b3a243ccd407
# ╟─3b119e3e-e9bf-4426-9a4e-509504a7a122
# ╠═c496f682-73c2-40a8-88bf-c5f8554b9e3c
# ╟─89565b78-3ce4-4f0e-9ee2-0d563d7f9e68
# ╠═bfd1d0ef-f5fc-4825-8e17-0be4075e1902
# ╟─cace5be5-7a28-42ea-a958-d8eb5cced10d
# ╠═077586e0-70a0-4322-a531-3c71b6eab95f
# ╟─230ed75d-1a16-49ea-9f92-876b69ffad70
# ╟─5a4c70b0-4f7c-4ef3-aabf-af1c46978637
# ╟─a7b2ef76-4626-4e89-9c78-98fa286923b3
# ╟─b53240b4-4fc7-4b07-acc2-b5381bcd6c86
# ╟─87f6de5e-ccbe-4295-a37f-123cfccaf3e8
# ╟─a6b6892e-c679-46e8-8553-65a3055fb472
# ╟─8493e2d6-7a73-4616-aa0b-11d7c8109c23
# ╟─b3f2f75f-ad9a-4e58-bb03-74de28379dbf
# ╟─348c030b-fd65-446c-8620-20a3f561aa81
# ╟─afa3ef46-b642-4f62-a1ca-31c4a456cb2b
# ╟─470ad9b2-d7be-41f6-9008-49fc4a245b06
# ╟─fb512cbd-9754-488b-89b7-91ef84dc350c
# ╟─5d0cee9b-73c6-4b7e-bb0f-6773be82f845
# ╟─c7da461d-1fc4-463e-bd46-900df67d1e03
# ╟─ac6cd3a1-f915-4ea9-9294-1158b8b3573f
# ╟─b01f5e40-e584-4615-9fbf-b4e9337649d9
# ╟─f0f5635b-897b-4ee0-ab8d-2a5e78b6c7d8
# ╠═a77a222e-aba3-4ac6-a5b0-a7551e3ccce4
# ╟─c66efe64-1130-42e1-bf43-a7c03baff579
# ╠═29089f61-955b-4f02-97a1-9e49ec796e96
# ╠═ae6e7a3f-4d2b-417f-aa66-af24a5803896
# ╟─6fe378c7-4970-4d12-8106-2ffcba89537d
# ╟─5eb27597-8fcf-4f35-8a49-818bdbce8951
# ╟─e52f724e-4530-4221-92ff-9f558d120db7
# ╠═5652b23c-c152-4a8b-ac7d-b33d782c0800
# ╟─34adc2d7-d7a6-4f62-b151-c71eff996074
# ╟─b2071081-ad56-43a8-8bf4-54802bfbc739
# ╟─a006eac2-59aa-43c1-b30a-8f7c2391ef27
# ╟─c38dbe18-a85a-4960-9c99-674196e0526d
# ╟─cb0d6593-5ca4-4afd-bddc-88506c4cf477
# ╟─afcdf0e6-4563-4bbb-a2db-6aa6c2d07b42
# ╟─495cbcdb-4120-4200-bfa6-5d1ad59308bb
# ╟─3e0f6675-3586-4f9f-8cec-cf87acae58f6
# ╟─46fae038-8319-4f88-b554-600ef7c762c8
# ╟─b6f43915-f5a4-471e-b018-c8c31d3185b9
# ╟─d9715727-6545-4f32-a9f4-f450ee98341d
# ╟─5499984f-c56d-4a84-aa6b-24d48ec3b786
# ╟─d9c4f0f6-f54e-4bd8-ba73-0797fbd23dde
# ╟─add07024-b7eb-445d-b4f4-6a6757c2ecd3
# ╟─7f67d656-bb55-4097-ba49-10140f9b3502
# ╟─7e47b3bc-a45c-4d84-9a58-d4f01d6ba99f
# ╟─d4c73d36-fc14-4968-9166-660e05cd2078
# ╟─bdea4838-cbd9-4947-bc74-b351658952c1
# ╟─291adf2b-f5d7-40da-9778-f2218045273d
# ╟─dcd13560-0f73-4aaa-b59a-b7961d8cdc63
# ╟─82a2c755-960e-4eb7-a55a-e333eb85fa5f
# ╠═4ac136d2-0413-4b4d-a990-9bf2a49ad482
# ╟─3bc3f9b4-c716-4655-a3cd-e83efbdbf4b9
# ╟─5ce15489-49a3-4942-b3dc-15a7629169c6
# ╟─292b1c34-b9d5-4468-98a9-4ee7a85d865b
# ╟─26fe310e-9ed8-4d09-bb82-a5bd39193dde
# ╟─63e8392d-c2b3-4a1a-b9f1-2acffe23ecb4
# ╟─3bed5d29-9973-4990-961a-53e53451854e
# ╟─9c5a5fcc-4655-4250-9214-1e819a862fa5
# ╟─8bfbcdd2-66b4-4884-8edd-71d3d8416a21
# ╟─9f0b3172-d9cd-4275-8b20-cb42e988cc0f
# ╟─a9818a71-fc9c-483a-9482-2f062232e73d
# ╟─6cee0cc7-2695-48df-932d-0128fcae87a3
# ╟─ad3c4297-138f-4bdd-b839-e85ead198ee7
# ╟─1503e906-0783-407d-a3f7-134d3c99c766
# ╠═a385a0ae-d2e0-4b4e-862c-e69ec5bd6843
# ╠═9fa87191-7c90-4738-a45a-acd929c8bd1b
# ╟─99e824b2-e12c-4090-87cb-45b3d042ee79
# ╟─916b9c85-e357-4d10-bba1-2b641a4dbc28
# ╟─3d73b76d-59e2-4249-9034-f9bbb2a1074b
# ╟─edf08bd1-dcd2-4129-a61f-c1e4f6bb09d9
# ╟─2dce7165-029d-43cc-8358-9a8df3f86d2a
# ╟─0b858694-3d54-4f05-8132-9bfcd302534a
# ╟─dde94490-9fdd-4417-90ae-0a6735eb505a
# ╟─c9942e2d-318d-48b8-b145-96425791cae5
# ╟─5811e2cb-4448-4b03-a812-ab284b583c86
# ╟─056d9341-78f1-4cb1-8a8e-a71ff246cf9d
# ╟─da6d64b3-cea6-42b4-9270-97e5df74c5cf
# ╟─18a64d4b-5741-44c8-ac1c-60e00f03828c
# ╟─d7e4c204-d21d-4119-a26f-124cf6ec5fa5
# ╟─0ce61106-cc15-44fa-a9a6-02ae2074154a
# ╟─6991c328-413a-4947-8542-a585e9010bca
# ╟─f7404380-0681-4466-ac61-a0f1dc4b94ef
# ╟─a7d4f49e-e317-44bd-bc77-a136e8182d59
# ╟─99e3138f-5fea-43e7-8f6b-195f2517c673
# ╟─c44af435-f632-4542-b914-fe83dcde78a5
# ╟─e3be2df9-6d5f-4541-ba35-ec65ebfeb4c3
# ╟─cc0f9f16-fd0b-4024-9fe7-744294d744c3
# ╟─1385bba0-7daf-4fe6-a9f8-9cc2fa29e600
# ╟─9b5ac60d-6732-4636-a2b2-08ab6f7297ad
# ╟─ecd1d9df-bec2-4d67-850b-1c263bfe2606
# ╟─1684b665-eae3-4e96-9f10-5909e09d9964
# ╟─2a8b4884-a1e7-4803-a2b1-6ce85955de93
# ╟─dad66466-0f3f-4908-92a4-50c5b20e5be5
# ╟─6ac7c37a-96a5-49d8-8fff-b76e381b6367
# ╟─f5fde724-21f9-4184-a317-8dfe75ec84ba
# ╟─e00c9d65-074e-4cb3-8e08-b32a58ee92c4
# ╟─1e98e928-68bd-4102-bdc8-2536b4615eaa
# ╟─1d7292a7-0dc6-4c0a-8e3c-fce761f3c499
# ╟─eeeb30c0-6b7d-42ff-93b8-dd647d72babe
# ╟─91fd4da1-abe0-4096-adcd-581d41fbfc33
# ╟─a81ad800-8dcc-4972-8245-5edf568aa611
# ╟─a2730369-1206-453d-930a-b6c779395cd5
# ╟─56bf8ed2-5786-4e51-b9c8-7a68a0690283
# ╟─e1c5c854-02c0-439c-9ba6-3f03be2a3d61
# ╟─f9d1d025-6b70-4a54-bc8a-7f7c5cad318a
# ╟─13f44f40-72d2-493f-a584-e17b6d0e102b
# ╟─f6d599fb-ac27-4c44-972f-28f300f43cdf
# ╟─ed9a0b2a-68fb-4996-81b2-60ba59248207
# ╟─b1ea211b-4ab8-49bd-96b7-dd8631cb689a
# ╟─47ac77f3-4f6d-4469-a2b7-edf35a049ed3
# ╟─51d0366d-3224-4a48-9e8e-7c8e4d41fd7e
# ╟─3a2b7a9b-0f24-4166-ba46-fc0b3df74f46
# ╟─9b68bbc2-d7f3-4bd7-9993-e4fe20363645
# ╟─5a8e44d3-ba71-4bb7-b6cc-509b08c9e1bb
# ╟─1b076f8f-f69f-40af-afe5-68749a409785
# ╟─1dfbe74a-61d7-48d8-823b-4d19efb3e6be
# ╟─960c426d-e69d-46f4-bc7c-a9c921b1a257
# ╟─29a9d34f-f239-43eb-9ce8-9c07c35d2d4e
# ╟─81536d10-9345-4889-b6d1-8fda40d2b0cb
# ╟─00000000-0000-0000-0000-000000000001
# ╟─00000000-0000-0000-0000-000000000002
